# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access

import re
from typing import Iterable

from azure.ai.ml._restclient.v2024_01_01_preview import (
    AzureMachineLearningWorkspaces as ServiceClient202401Preview,
)
from azure.ai.ml._restclient.v2024_01_01_preview.models import (
    KeyType,
    RegenerateEndpointKeysRequest,
)
from azure.ai.ml._scope_dependent_operations import (
    OperationConfig,
    OperationsContainer,
    OperationScope,
    _ScopeDependentOperations,
)
from azure.ai.ml._telemetry import ActivityType, monitor_with_activity
from azure.ai.ml._utils._experimental import experimental
from azure.ai.ml._utils._logger_utils import OpsLogger
from azure.ai.ml.constants._common import REGISTRY_VERSION_PATTERN, AzureMLResourceType
from azure.ai.ml.constants._endpoint import EndpointKeyType
from azure.ai.ml.entities._autogen_entities.models import ServerlessEndpoint
from azure.ai.ml.entities._endpoint.online_endpoint import EndpointAuthKeys
from azure.ai.ml.exceptions import (
    ErrorCategory,
    ErrorTarget,
    ValidationErrorType,
    ValidationException,
)
from azure.core.polling import LROPoller

ops_logger = OpsLogger(__name__)
module_logger = ops_logger.module_logger


class ServerlessEndpointOperations(_ScopeDependentOperations):
    """ServerlessEndpointOperations.

    You should not instantiate this class directly. Instead, you should
    create an MLClient instance that instantiates it for you and
    attaches it as an attribute.
    """

    def __init__(
        self,
        operation_scope: OperationScope,
        operation_config: OperationConfig,
        service_client: ServiceClient202401Preview,
        all_operations: OperationsContainer,
    ):
        super().__init__(operation_scope, operation_config)
        ops_logger.update_filter()
        self._service_client = service_client.serverless_endpoints
        self._marketplace_subscriptions = service_client.marketplace_subscriptions
        self._all_operations = all_operations

    def _get_workspace_location(self) -> str:
        return str(
            self._all_operations.all_operations[AzureMLResourceType.WORKSPACE].get(self._workspace_name).location
        )

    @experimental
    @monitor_with_activity(ops_logger, "ServerlessEndpoint.BeginCreateOrUpdate", ActivityType.PUBLICAPI)
    def begin_create_or_update(self, endpoint: ServerlessEndpoint, **kwargs) -> LROPoller[ServerlessEndpoint]:
        """Create or update a serverless endpoint.

        :param endpoint: The serverless endpoint entity.
        :type endpoint: ~azure.ai.ml.entities.ServerlessEndpoint
        :raises ~azure.ai.ml.exceptions.ValidationException: Raised if ServerlessEndpoint cannot be
            successfully validated. Details will be provided in the error message.
        :return: A poller to track the operation status
        :rtype: ~azure.core.polling.LROPoller[~azure.ai.ml.entities.ServerlessEndpoint]
        """
        if not endpoint.location:
            endpoint.location = self._get_workspace_location()
        if re.match(REGISTRY_VERSION_PATTERN, endpoint.model_id):
            msg = (
                "The given model_id {} points to a specific model version, which is not supported. "
                "Please provide a model_id without the version information."
            )
            raise ValidationException(
                message=msg.format(endpoint.model_id),
                no_personal_data_message="Invalid model_id given for serverless endpoint",
                target=ErrorTarget.SERVERLESS_ENDPOINT,
                error_category=ErrorCategory.USER_ERROR,
                error_type=ValidationErrorType.INVALID_VALUE,
            )
        return self._service_client.begin_create_or_update(
            self._resource_group_name,
            self._workspace_name,
            endpoint.name,
            endpoint._to_rest_object(),  # type: ignore
            cls=(
                lambda response, deserialized, headers: ServerlessEndpoint._from_rest_object(  # type: ignore
                    deserialized
                )
            ),
            **kwargs,
        )

    @experimental
    @monitor_with_activity(ops_logger, "ServerlessEndpoint.Get", ActivityType.PUBLICAPI)
    def get(self, name: str, **kwargs) -> ServerlessEndpoint:
        """Get a Serverless Endpoint resource.

        :param name: Name of the serverless endpoint.
        :type name: str
        :return: Serverless endpoint object retrieved from the service.
        :rtype: ~azure.ai.ml.entities.ServerlessEndpoint
        """
        return self._service_client.get(
            self._resource_group_name,
            self._workspace_name,
            name,
            cls=(
                lambda response, deserialized, headers: ServerlessEndpoint._from_rest_object(  # type: ignore
                    deserialized
                )
            ),
            **kwargs,
        )

    @experimental
    @monitor_with_activity(ops_logger, "ServerlessEndpoint.list", ActivityType.PUBLICAPI)
    def list(self, **kwargs) -> Iterable[ServerlessEndpoint]:
        """List serverless endpoints of the workspace.

        :return: A list of serverless endpoints
        :rtype: ~typing.Iterable[~azure.ai.ml.entities.ServerlessEndpoint]
        """
        return self._service_client.list(
            self._resource_group_name,
            self._workspace_name,
            cls=lambda objs: [ServerlessEndpoint._from_rest_object(obj) for obj in objs],  # type: ignore
            **kwargs,
        )

    @experimental
    @monitor_with_activity(ops_logger, "ServerlessEndpoint.BeginDelete", ActivityType.PUBLICAPI)
    def begin_delete(self, name: str, **kwargs) -> LROPoller[None]:
        """Delete a Serverless Endpoint.

        :param name: Name of the serverless endpoint.
        :type name: str
        :return: A poller to track the operation status.
        :rtype: ~azure.core.polling.LROPoller[None]
        """
        return self._service_client.begin_delete(
            self._resource_group_name,
            self._workspace_name,
            name,
            **kwargs,
        )

    @experimental
    @monitor_with_activity(ops_logger, "ServerlessEndpoint.GetKeys", ActivityType.PUBLICAPI)
    def get_keys(self, name: str, **kwargs) -> EndpointAuthKeys:
        """Get serveless endpoint auth keys.

        :param name: The serverless endpoint name
        :type name: str
        :return: Returns the keys of the serverless endpoint
        :rtype: ~azure.ai.ml.entities.EndpointAuthKeys
        """
        return self._service_client.list_keys(
            self._resource_group_name,
            self._workspace_name,
            name,
            cls=lambda response, deserialized, headers: EndpointAuthKeys._from_rest_object(deserialized),
            **kwargs,
        )

    @experimental
    @monitor_with_activity(ops_logger, "ServerlessEndpoint.BeginRegenerateKeys", ActivityType.PUBLICAPI)
    def begin_regenerate_keys(
        self,
        name: str,
        *,
        key_type: str = EndpointKeyType.PRIMARY_KEY_TYPE,
        **kwargs,
    ) -> LROPoller[EndpointAuthKeys]:
        """Regenerate keys for a serverless endpoint.

        :param name: The endpoint name.
        :type name: str
        :keyword key_type: One of "primary", "secondary". Defaults to "primary".
        :paramtype key_type: str
        :raises ~azure.ai.ml.exceptions.ValidationException: Raised if key_type is not "primary"
            or "secondary"
        :return: A poller to track the operation status.
        :rtype: ~azure.core.polling.LROPoller[EndpointAuthKeys]
        """
        keys = self.get_keys(
            name=name,
        )
        if key_type.lower() == EndpointKeyType.PRIMARY_KEY_TYPE:
            key_request = RegenerateEndpointKeysRequest(key_type=KeyType.Primary, key_value=keys.primary_key)
        elif key_type.lower() == EndpointKeyType.SECONDARY_KEY_TYPE:
            key_request = RegenerateEndpointKeysRequest(key_type=KeyType.Secondary, key_value=keys.secondary_key)
        else:
            msg = "Key type must be 'primary' or 'secondary'."
            raise ValidationException(
                message=msg,
                target=ErrorTarget.SERVERLESS_ENDPOINT,
                no_personal_data_message=msg,
                error_category=ErrorCategory.USER_ERROR,
                error_type=ValidationErrorType.INVALID_VALUE,
            )

        return self._service_client.begin_regenerate_keys(
            resource_group_name=self._resource_group_name,
            workspace_name=self._workspace_name,
            endpoint_name=name,
            body=key_request,
            cls=lambda response, deserialized, headers: EndpointAuthKeys._from_rest_object(deserialized),
            **kwargs,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument
from marshmallow import fields
from marshmallow.decorators import post_load

from azure.ai.ml._schema.core.fields import NestedField, StringTransformedEnum
from azure.ai.ml._utils.utils import camel_to_snake
from azure.ai.ml._vendor.azure_resources.models._resource_management_client_enums import ResourceIdentityType
from azure.ai.ml.entities._credentials import ManagedIdentityConfiguration

from ..core.schema import PathAwareSchema


class ComputeSchema(PathAwareSchema):
    name = fields.Str(required=True)
    id = fields.Str(dump_only=True)
    type = fields.Str()
    location = fields.Str()
    description = fields.Str()
    provisioning_errors = fields.Str(dump_only=True)
    created_on = fields.Str(dump_only=True)
    provisioning_state = fields.Str(dump_only=True)
    resource_id = fields.Str()
    tags = fields.Dict(keys=fields.Str(), values=fields.Str())


class NetworkSettingsSchema(PathAwareSchema):
    vnet_name = fields.Str()
    subnet = fields.Str()
    public_ip_address = fields.Str(dump_only=True)
    private_ip_address = fields.Str(dump_only=True)

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities import NetworkSettings

        return NetworkSettings(**data)


class UserAssignedIdentitySchema(PathAwareSchema):
    resource_id = fields.Str()
    principal_id = fields.Str(dump_only=True)
    client_id = fields.Str(dump_only=True)
    tenant_id = fields.Str(dump_only=True)

    @post_load
    def make(self, data, **kwargs):
        return ManagedIdentityConfiguration(**data)


class IdentitySchema(PathAwareSchema):
    type = StringTransformedEnum(
        allowed_values=[
            ResourceIdentityType.SYSTEM_ASSIGNED,
            ResourceIdentityType.USER_ASSIGNED,
            ResourceIdentityType.NONE,
            ResourceIdentityType.SYSTEM_ASSIGNED_USER_ASSIGNED,
        ],
        casing_transform=camel_to_snake,
        metadata={"description": "resource identity type."},
    )
    user_assigned_identities = fields.List(NestedField(UserAssignedIdentitySchema))
    principal_id = fields.Str(dump_only=True)
    tenant_id = fields.Str(dump_only=True)

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities import IdentityConfiguration

        user_assigned_identities_list = []
        user_assigned_identities = data.pop("user_assigned_identities", None)
        if user_assigned_identities:
            for identity in user_assigned_identities:
                user_assigned_identities_list.append(
                    ManagedIdentityConfiguration(
                        resource_id=identity.get("resource_id", None),
                        client_id=identity.get("client_id", None),
                        object_id=identity.get("object_id", None),
                    )
                )
            data["user_assigned_identities"] = user_assigned_identities_list
        return IdentityConfiguration(**data)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from marshmallow import fields
from azure.ai.ml._schema.core.schema import YamlFileSchema


class FeatureWindowSchema(YamlFileSchema):
    feature_window_end = fields.Str()
    feature_window_start = fields.Str()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import os
import argparse
import datetime
from pathlib import Path
import yaml
from mltable import load


def parse_args():
    # setup arg parser
    parser = argparse.ArgumentParser()

    # add arguments
    parser.add_argument("--train_data", type=str)
    parser.add_argument("--validation_data", type=str)
    parser.add_argument("--preprocessed_train_data", type=str)
    parser.add_argument("--preprocessed_validation_data", type=str)
    # parse args
    args = parser.parse_args()
    print("args received ", args)
    # return args
    return args


def get_preprocessed_data(text):
    """
    Do preprocessing as needed
    Currently we are just passing text file as it is
    """
    return text


def main(args):
    """
    Preprocessing of training/validation data
    """
    train_data_path = os.path.join(args.train_data, "train.txt")
    with open(train_data_path) as f:
        train_data = f.read()
    preprocessed_train_data = get_preprocessed_data(train_data)

    # write preprocessed train txt file
    preprocessed_train_data_path = os.path.join(args.preprocessed_train_data, "train.txt")
    with open(preprocessed_train_data_path, "w") as f:
        f.write(preprocessed_train_data)

    validation_data_path = os.path.join(args.validation_data, "valid.txt")
    with open(validation_data_path) as f:
        validation_data = f.read()
    preprocessed_validation_data = get_preprocessed_data(validation_data)

    # write preprocessed validation txt file
    preprocessed_validation_data_path = os.path.join(args.preprocessed_validation_data, "valid.txt")
    with open(preprocessed_validation_data_path, "w") as f:
        f.write(preprocessed_validation_data)

    # Write MLTable yaml file as well in output folder
    # Since in this example we are not doing any preprocessing, we are just copying same yaml file from input,change it if needed

    # read and write MLModel yaml file for train data
    train_data_mltable_path = os.path.join(args.train_data, "MLTable")
    preprocessed_train_data_mltable_path = os.path.join(args.preprocessed_train_data, "MLTable")
    with open(train_data_mltable_path, "r") as file:
        yaml_file = yaml.safe_load(file)
    with open(preprocessed_train_data_mltable_path, "w") as file:
        yaml.dump(yaml_file, file)

    # read and write MLModel yaml file for validation data
    validation_data_mltable_path = os.path.join(args.validation_data, "MLTable")
    preprocessed_validation_data_mltable_path = os.path.join(args.preprocessed_validation_data, "MLTable")
    with open(validation_data_mltable_path, "r") as file:
        yaml_file = yaml.safe_load(file)
    with open(preprocessed_validation_data_mltable_path, "w") as file:
        yaml.dump(yaml_file, file)


# run script
if __name__ == "__main__":
    # parse args
    args = parse_args()

    # run main function
    main(args)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import TYPE_CHECKING, Any, Awaitable

from msrest import Deserializer, Serializer

from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient

from .. import models
from ._configuration import AzureMachineLearningWorkspacesConfiguration
from .operations import (
    BatchDeploymentsOperations,
    BatchEndpointsOperations,
    CapacityReservationGroupsOperations,
    CodeContainersOperations,
    CodeVersionsOperations,
    ComponentContainersOperations,
    ComponentVersionsOperations,
    ComputeOperations,
    DataContainersOperations,
    DatastoresOperations,
    DataVersionsOperations,
    EndpointDeploymentOperations,
    EndpointOperations,
    EnvironmentContainersOperations,
    EnvironmentVersionsOperations,
    FeaturesetContainersOperations,
    FeaturesetVersionsOperations,
    FeaturesOperations,
    FeaturestoreEntityContainersOperations,
    FeaturestoreEntityVersionsOperations,
    InferenceEndpointsOperations,
    InferenceGroupsOperations,
    InferencePoolsOperations,
    JobsOperations,
    LabelingJobsOperations,
    ManagedNetworkProvisionsOperations,
    ManagedNetworkSettingsRuleOperations,
    MarketplaceSubscriptionsOperations,
    ModelContainersOperations,
    ModelVersionsOperations,
    OnlineDeploymentsOperations,
    OnlineEndpointsOperations,
    Operations,
    PrivateEndpointConnectionsOperations,
    PrivateLinkResourcesOperations,
    QuotasOperations,
    RegistriesOperations,
    RegistryCodeContainersOperations,
    RegistryCodeVersionsOperations,
    RegistryComponentContainersOperations,
    RegistryComponentVersionsOperations,
    RegistryDataContainersOperations,
    RegistryDataReferencesOperations,
    RegistryDataVersionsOperations,
    RegistryEnvironmentContainersOperations,
    RegistryEnvironmentVersionsOperations,
    RegistryModelContainersOperations,
    RegistryModelVersionsOperations,
    SchedulesOperations,
    ServerlessEndpointsOperations,
    UsagesOperations,
    VirtualMachineSizesOperations,
    WorkspaceConnectionsOperations,
    WorkspaceFeaturesOperations,
    WorkspacesOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential

class AzureMachineLearningWorkspaces:    # pylint: disable=too-many-instance-attributes
    """These APIs allow end users to operate on Azure Machine Learning Workspace resources.

    :ivar usages: UsagesOperations operations
    :vartype usages: azure.mgmt.machinelearningservices.aio.operations.UsagesOperations
    :ivar virtual_machine_sizes: VirtualMachineSizesOperations operations
    :vartype virtual_machine_sizes:
     azure.mgmt.machinelearningservices.aio.operations.VirtualMachineSizesOperations
    :ivar quotas: QuotasOperations operations
    :vartype quotas: azure.mgmt.machinelearningservices.aio.operations.QuotasOperations
    :ivar compute: ComputeOperations operations
    :vartype compute: azure.mgmt.machinelearningservices.aio.operations.ComputeOperations
    :ivar registries: RegistriesOperations operations
    :vartype registries: azure.mgmt.machinelearningservices.aio.operations.RegistriesOperations
    :ivar workspace_features: WorkspaceFeaturesOperations operations
    :vartype workspace_features:
     azure.mgmt.machinelearningservices.aio.operations.WorkspaceFeaturesOperations
    :ivar capacity_reservation_groups: CapacityReservationGroupsOperations operations
    :vartype capacity_reservation_groups:
     azure.mgmt.machinelearningservices.aio.operations.CapacityReservationGroupsOperations
    :ivar registry_code_containers: RegistryCodeContainersOperations operations
    :vartype registry_code_containers:
     azure.mgmt.machinelearningservices.aio.operations.RegistryCodeContainersOperations
    :ivar registry_code_versions: RegistryCodeVersionsOperations operations
    :vartype registry_code_versions:
     azure.mgmt.machinelearningservices.aio.operations.RegistryCodeVersionsOperations
    :ivar registry_component_containers: RegistryComponentContainersOperations operations
    :vartype registry_component_containers:
     azure.mgmt.machinelearningservices.aio.operations.RegistryComponentContainersOperations
    :ivar registry_component_versions: RegistryComponentVersionsOperations operations
    :vartype registry_component_versions:
     azure.mgmt.machinelearningservices.aio.operations.RegistryComponentVersionsOperations
    :ivar registry_data_containers: RegistryDataContainersOperations operations
    :vartype registry_data_containers:
     azure.mgmt.machinelearningservices.aio.operations.RegistryDataContainersOperations
    :ivar registry_data_versions: RegistryDataVersionsOperations operations
    :vartype registry_data_versions:
     azure.mgmt.machinelearningservices.aio.operations.RegistryDataVersionsOperations
    :ivar registry_data_references: RegistryDataReferencesOperations operations
    :vartype registry_data_references:
     azure.mgmt.machinelearningservices.aio.operations.RegistryDataReferencesOperations
    :ivar registry_environment_containers: RegistryEnvironmentContainersOperations operations
    :vartype registry_environment_containers:
     azure.mgmt.machinelearningservices.aio.operations.RegistryEnvironmentContainersOperations
    :ivar registry_environment_versions: RegistryEnvironmentVersionsOperations operations
    :vartype registry_environment_versions:
     azure.mgmt.machinelearningservices.aio.operations.RegistryEnvironmentVersionsOperations
    :ivar marketplace_subscriptions: MarketplaceSubscriptionsOperations operations
    :vartype marketplace_subscriptions:
     azure.mgmt.machinelearningservices.aio.operations.MarketplaceSubscriptionsOperations
    :ivar registry_model_containers: RegistryModelContainersOperations operations
    :vartype registry_model_containers:
     azure.mgmt.machinelearningservices.aio.operations.RegistryModelContainersOperations
    :ivar registry_model_versions: RegistryModelVersionsOperations operations
    :vartype registry_model_versions:
     azure.mgmt.machinelearningservices.aio.operations.RegistryModelVersionsOperations
    :ivar batch_endpoints: BatchEndpointsOperations operations
    :vartype batch_endpoints:
     azure.mgmt.machinelearningservices.aio.operations.BatchEndpointsOperations
    :ivar batch_deployments: BatchDeploymentsOperations operations
    :vartype batch_deployments:
     azure.mgmt.machinelearningservices.aio.operations.BatchDeploymentsOperations
    :ivar code_containers: CodeContainersOperations operations
    :vartype code_containers:
     azure.mgmt.machinelearningservices.aio.operations.CodeContainersOperations
    :ivar code_versions: CodeVersionsOperations operations
    :vartype code_versions:
     azure.mgmt.machinelearningservices.aio.operations.CodeVersionsOperations
    :ivar component_containers: ComponentContainersOperations operations
    :vartype component_containers:
     azure.mgmt.machinelearningservices.aio.operations.ComponentContainersOperations
    :ivar component_versions: ComponentVersionsOperations operations
    :vartype component_versions:
     azure.mgmt.machinelearningservices.aio.operations.ComponentVersionsOperations
    :ivar data_containers: DataContainersOperations operations
    :vartype data_containers:
     azure.mgmt.machinelearningservices.aio.operations.DataContainersOperations
    :ivar data_versions: DataVersionsOperations operations
    :vartype data_versions:
     azure.mgmt.machinelearningservices.aio.operations.DataVersionsOperations
    :ivar datastores: DatastoresOperations operations
    :vartype datastores: azure.mgmt.machinelearningservices.aio.operations.DatastoresOperations
    :ivar environment_containers: EnvironmentContainersOperations operations
    :vartype environment_containers:
     azure.mgmt.machinelearningservices.aio.operations.EnvironmentContainersOperations
    :ivar environment_versions: EnvironmentVersionsOperations operations
    :vartype environment_versions:
     azure.mgmt.machinelearningservices.aio.operations.EnvironmentVersionsOperations
    :ivar featureset_containers: FeaturesetContainersOperations operations
    :vartype featureset_containers:
     azure.mgmt.machinelearningservices.aio.operations.FeaturesetContainersOperations
    :ivar features: FeaturesOperations operations
    :vartype features: azure.mgmt.machinelearningservices.aio.operations.FeaturesOperations
    :ivar featureset_versions: FeaturesetVersionsOperations operations
    :vartype featureset_versions:
     azure.mgmt.machinelearningservices.aio.operations.FeaturesetVersionsOperations
    :ivar featurestore_entity_containers: FeaturestoreEntityContainersOperations operations
    :vartype featurestore_entity_containers:
     azure.mgmt.machinelearningservices.aio.operations.FeaturestoreEntityContainersOperations
    :ivar featurestore_entity_versions: FeaturestoreEntityVersionsOperations operations
    :vartype featurestore_entity_versions:
     azure.mgmt.machinelearningservices.aio.operations.FeaturestoreEntityVersionsOperations
    :ivar inference_pools: InferencePoolsOperations operations
    :vartype inference_pools:
     azure.mgmt.machinelearningservices.aio.operations.InferencePoolsOperations
    :ivar inference_endpoints: InferenceEndpointsOperations operations
    :vartype inference_endpoints:
     azure.mgmt.machinelearningservices.aio.operations.InferenceEndpointsOperations
    :ivar inference_groups: InferenceGroupsOperations operations
    :vartype inference_groups:
     azure.mgmt.machinelearningservices.aio.operations.InferenceGroupsOperations
    :ivar jobs: JobsOperations operations
    :vartype jobs: azure.mgmt.machinelearningservices.aio.operations.JobsOperations
    :ivar labeling_jobs: LabelingJobsOperations operations
    :vartype labeling_jobs:
     azure.mgmt.machinelearningservices.aio.operations.LabelingJobsOperations
    :ivar model_containers: ModelContainersOperations operations
    :vartype model_containers:
     azure.mgmt.machinelearningservices.aio.operations.ModelContainersOperations
    :ivar model_versions: ModelVersionsOperations operations
    :vartype model_versions:
     azure.mgmt.machinelearningservices.aio.operations.ModelVersionsOperations
    :ivar online_endpoints: OnlineEndpointsOperations operations
    :vartype online_endpoints:
     azure.mgmt.machinelearningservices.aio.operations.OnlineEndpointsOperations
    :ivar online_deployments: OnlineDeploymentsOperations operations
    :vartype online_deployments:
     azure.mgmt.machinelearningservices.aio.operations.OnlineDeploymentsOperations
    :ivar schedules: SchedulesOperations operations
    :vartype schedules: azure.mgmt.machinelearningservices.aio.operations.SchedulesOperations
    :ivar serverless_endpoints: ServerlessEndpointsOperations operations
    :vartype serverless_endpoints:
     azure.mgmt.machinelearningservices.aio.operations.ServerlessEndpointsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.machinelearningservices.aio.operations.Operations
    :ivar workspaces: WorkspacesOperations operations
    :vartype workspaces: azure.mgmt.machinelearningservices.aio.operations.WorkspacesOperations
    :ivar workspace_connections: WorkspaceConnectionsOperations operations
    :vartype workspace_connections:
     azure.mgmt.machinelearningservices.aio.operations.WorkspaceConnectionsOperations
    :ivar endpoint_deployment: EndpointDeploymentOperations operations
    :vartype endpoint_deployment:
     azure.mgmt.machinelearningservices.aio.operations.EndpointDeploymentOperations
    :ivar endpoint: EndpointOperations operations
    :vartype endpoint: azure.mgmt.machinelearningservices.aio.operations.EndpointOperations
    :ivar managed_network_settings_rule: ManagedNetworkSettingsRuleOperations operations
    :vartype managed_network_settings_rule:
     azure.mgmt.machinelearningservices.aio.operations.ManagedNetworkSettingsRuleOperations
    :ivar private_endpoint_connections: PrivateEndpointConnectionsOperations operations
    :vartype private_endpoint_connections:
     azure.mgmt.machinelearningservices.aio.operations.PrivateEndpointConnectionsOperations
    :ivar private_link_resources: PrivateLinkResourcesOperations operations
    :vartype private_link_resources:
     azure.mgmt.machinelearningservices.aio.operations.PrivateLinkResourcesOperations
    :ivar managed_network_provisions: ManagedNetworkProvisionsOperations operations
    :vartype managed_network_provisions:
     azure.mgmt.machinelearningservices.aio.operations.ManagedNetworkProvisionsOperations
    :param credential: Credential needed for the client to connect to Azure.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID of the target subscription.
    :type subscription_id: str
    :param base_url: Service URL. Default value is 'https://management.azure.com'.
    :type base_url: str
    :keyword api_version: Api Version. The default value is "2024-01-01-preview". Note that
     overriding this default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = AzureMachineLearningWorkspacesConfiguration(credential=credential, subscription_id=subscription_id, **kwargs)
        self._client = AsyncARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.usages = UsagesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.virtual_machine_sizes = VirtualMachineSizesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.quotas = QuotasOperations(self._client, self._config, self._serialize, self._deserialize)
        self.compute = ComputeOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registries = RegistriesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.workspace_features = WorkspaceFeaturesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.capacity_reservation_groups = CapacityReservationGroupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_code_containers = RegistryCodeContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_code_versions = RegistryCodeVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_component_containers = RegistryComponentContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_component_versions = RegistryComponentVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_data_containers = RegistryDataContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_data_versions = RegistryDataVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_data_references = RegistryDataReferencesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_environment_containers = RegistryEnvironmentContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_environment_versions = RegistryEnvironmentVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.marketplace_subscriptions = MarketplaceSubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_model_containers = RegistryModelContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.registry_model_versions = RegistryModelVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.batch_endpoints = BatchEndpointsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.batch_deployments = BatchDeploymentsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.code_containers = CodeContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.code_versions = CodeVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.component_containers = ComponentContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.component_versions = ComponentVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.data_containers = DataContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.data_versions = DataVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.datastores = DatastoresOperations(self._client, self._config, self._serialize, self._deserialize)
        self.environment_containers = EnvironmentContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.environment_versions = EnvironmentVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.featureset_containers = FeaturesetContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.features = FeaturesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.featureset_versions = FeaturesetVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.featurestore_entity_containers = FeaturestoreEntityContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.featurestore_entity_versions = FeaturestoreEntityVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.inference_pools = InferencePoolsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.inference_endpoints = InferenceEndpointsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.inference_groups = InferenceGroupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.jobs = JobsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.labeling_jobs = LabelingJobsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.model_containers = ModelContainersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.model_versions = ModelVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.online_endpoints = OnlineEndpointsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.online_deployments = OnlineDeploymentsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.schedules = SchedulesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.serverless_endpoints = ServerlessEndpointsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.workspaces = WorkspacesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.workspace_connections = WorkspaceConnectionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.endpoint_deployment = EndpointDeploymentOperations(self._client, self._config, self._serialize, self._deserialize)
        self.endpoint = EndpointOperations(self._client, self._config, self._serialize, self._deserialize)
        self.managed_network_settings_rule = ManagedNetworkSettingsRuleOperations(self._client, self._config, self._serialize, self._deserialize)
        self.private_endpoint_connections = PrivateEndpointConnectionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.private_link_resources = PrivateLinkResourcesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.managed_network_provisions = ManagedNetworkProvisionsOperations(self._client, self._config, self._serialize, self._deserialize)


    def _send_request(
        self,
        request: HttpRequest,
        **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/python/protocol/quickstart

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> "AzureMachineLearningWorkspaces":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details) -> None:
        await self._client.__aexit__(*exc_details)



from pathlib import Path

from azure.ai.ml import load_component
from azure.ai.ml.dsl import pipeline
from azure.ai.ml.dsl._group_decorator import group

command_component_func = load_component(Path(__file__).parent / "command_component.yml")
pipeline_component_func = load_component(Path(__file__).parent / "pipeline_component.yml")


@group
class ParameterGroup:
    int_param: int
    str_param: str


@pipeline
def pipeline_func(pg: ParameterGroup):
    # init job consume items from group
    init_job = command_component_func(  # noqa: F841
        component_in_number=pg.int_param,
        component_in_string=pg.str_param,
    )
    # finalize job consume parameter group
    finalize_job = pipeline_component_func(pg=pg)  # noqa: F841
    # execution jobs
    take = command_component_func(  # noqa: F841
        component_in_number=pg.int_param,
        component_in_string=pg.str_param,
    )


pipeline_job = pipeline_func(pg=ParameterGroup(int_param=1, str_param="a"))
pipeline_job.settings.default_compute = "cpu-cluster"
pipeline_job.settings.on_init = "init_job"
pipeline_job.settings.on_finalize = "finalize_job"



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Optional

from azure.ai.ml._restclient.v2023_04_01_preview.models import ImageLimitSettings as RestImageLimitSettings
from azure.ai.ml._utils.utils import from_iso_duration_format_mins, to_iso_duration_format_mins
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class ImageLimitSettings(RestTranslatableMixin):
    r"""Limit settings for AutoML Image Verticals.

    ImageLimitSettings is a class that contains the following parameters:  max_concurrent_trials, max_trials, and \
    timeout_minutes.

    This is an optional configuration method to configure limits parameters such as timeouts etc.

        .. note::

            The number of concurrent runs is gated on the resources available in the specified compute target.
            Ensure that the compute target has the available resources for the desired concurrency.

    :keyword max_concurrent_trials: Maximum number of concurrent AutoML iterations, defaults to None.
    :paramtype  max_concurrent_trials: typing.Optional[int]
    :keyword max_trials: Represents the maximum number of trials (children jobs).
    :paramtype  max_trials: typing.Optional[int]
    :keyword timeout_minutes: AutoML job timeout. Defaults to None
    :paramtype  timeout_minutes: typing.Optional[int]
    :raises ValueError: If max_concurrent_trials is not None and is not a positive integer.
    :raises ValueError: If max_trials is not None and is not a positive integer.
    :raises ValueError: If timeout_minutes is not None and is not a positive integer.
    :return: ImageLimitSettings object.
    :rtype: ImageLimitSettings

    .. tip::
        It's a good practice to match max_concurrent_trials count with the number of nodes in the cluster.
        For example, if you have a cluster with 4 nodes, set max_concurrent_trials to 4.

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_automl_image.py
                :start-after: [START automl.automl_image_job.image_limit_settings]
                :end-before: [END automl.automl_image_job.image_limit_settings]
                :language: python
                :dedent: 8
                :caption: Defining the limit settings for an automl image job.
    """

    def __init__(
        self,
        *,
        max_concurrent_trials: Optional[int] = None,
        max_trials: Optional[int] = None,
        timeout_minutes: Optional[int] = None,
    ) -> None:
        self.max_concurrent_trials = max_concurrent_trials
        self.max_trials = max_trials
        self.timeout_minutes = timeout_minutes

    def _to_rest_object(self) -> RestImageLimitSettings:
        """Convert ImageLimitSettings objects to a rest object.

        :return: A rest object of ImageLimitSettings objects.
        :rtype: RestImageLimitSettings
        """
        return RestImageLimitSettings(
            max_concurrent_trials=self.max_concurrent_trials,
            max_trials=self.max_trials,
            timeout=to_iso_duration_format_mins(self.timeout_minutes),
        )

    @classmethod
    def _from_rest_object(cls, obj: RestImageLimitSettings) -> "ImageLimitSettings":
        """Convert the rest object to a dict containing items to init the ImageLimitSettings objects.

        :param obj: Limit settings for the AutoML job in Rest format.
        :type obj: RestImageLimitSettings
        :return: Limit settings for an AutoML Image Vertical.
        :rtype: ImageLimitSettings
        """
        return cls(
            max_concurrent_trials=obj.max_concurrent_trials,
            max_trials=obj.max_trials,
            timeout_minutes=from_iso_duration_format_mins(obj.timeout),
        )

    def __eq__(self, other: object) -> bool:
        """Check equality between two ImageLimitSettings objects.

        This method check instances equality and returns True if both of
            the instances have the same attributes with the same values.

        :param other: Any object
        :type other: object
        :return: True or False
        :rtype: bool
        """
        if not isinstance(other, ImageLimitSettings):
            return NotImplemented

        return (
            self.max_concurrent_trials == other.max_concurrent_trials
            and self.max_trials == other.max_trials
            and self.timeout_minutes == other.timeout_minutes
        )

    def __ne__(self, other: object) -> bool:
        """Check inequality between two ImageLimitSettings objects.

        :param other: Any object
        :type other: object
        :return: True or False
        :rtype: bool
        """
        return not self.__eq__(other)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from marshmallow import fields

from azure.ai.ml._schema.automl.automl_vertical import AutoMLVerticalSchema
from azure.ai.ml._schema.automl.featurization_settings import NlpFeaturizationSettingsSchema
from azure.ai.ml._schema.automl.nlp_vertical.nlp_fixed_parameters import NlpFixedParametersSchema
from azure.ai.ml._schema.automl.nlp_vertical.nlp_parameter_subspace import NlpParameterSubspaceSchema
from azure.ai.ml._schema.automl.nlp_vertical.nlp_sweep_settings import NlpSweepSettingsSchema
from azure.ai.ml._schema.automl.nlp_vertical.nlp_vertical_limit_settings import NlpLimitsSchema
from azure.ai.ml._schema.core.fields import NestedField, UnionField
from azure.ai.ml._schema.job.input_output_entry import MLTableInputSchema
from azure.ai.ml.constants._job.automl import AutoMLConstants


class NlpVerticalSchema(AutoMLVerticalSchema):
    limits = NestedField(NlpLimitsSchema())
    sweep = NestedField(NlpSweepSettingsSchema())
    training_parameters = NestedField(NlpFixedParametersSchema())
    search_space = fields.List(NestedField(NlpParameterSubspaceSchema()))
    featurization = NestedField(NlpFeaturizationSettingsSchema(), data_key=AutoMLConstants.FEATURIZATION_YAML)
    validation_data = UnionField([NestedField(MLTableInputSchema)])



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

import logging
from typing import Any

from marshmallow import post_load

from azure.ai.ml._schema.job.import_job import ImportJobSchema

module_logger = logging.getLogger(__name__)


class PipelineImportJobSchema(ImportJobSchema):
    class Meta:
        exclude = ["compute"]  # compute property not applicable to import job

    @post_load
    def make(self, data: Any, **kwargs: Any):
        from azure.ai.ml.entities._job.import_job import ImportJob

        return ImportJob(**data)



import random

import pytest
import platform
import sys

from azure.ai.ml import MLClient, load_online_deployment, load_online_endpoint
from azure.ai.ml.entities import OnlineDeployment, OnlineEndpoint
from azure.ai.ml.entities._assets._artifacts.code import Code
from azure.ai.ml.entities._assets._artifacts.model import Model
from azure.ai.ml.entities._assets.environment import Environment
from azure.core.exceptions import ResourceNotFoundError


@pytest.fixture
def mir_endpoint_name() -> str:
    return f"mir-test-{str(random.randint(1, 10000000))}"


@pytest.fixture
def endpoint_mir_yaml() -> str:
    return "tests/test_configs/deployments/online/simple_online_endpoint_mir.yaml"


@pytest.fixture
def deployment_create_yaml() -> str:
    return "./tests/test_configs/deployments/online/online_deployment_2.yaml"


@pytest.fixture
def deployment_update_file() -> str:
    return "./tests/test_configs/deployments/online/online_deployment_2.yaml"


@pytest.fixture
def mir_update_file() -> str:
    return "./tests/test_configs/online/managed/canary-declarative-flow/6-delete-blue.yaml"


@pytest.fixture
def request_file() -> str:
    return "./tests/test_configs/deployments/model-1/sample-request.json"


@pytest.mark.e2etest
@pytest.mark.local_endpoint_local_assets
@pytest.mark.skipif(
    platform.python_implementation() == "PyPy" or sys.platform.startswith("darwin"),
    reason="Skipping for PyPy and macOS as docker installation is not supported and skipped in dev_requirement.txt",
)
def test_local_endpoint_mir_e2e(
    endpoint_mir_yaml: str,
    mir_endpoint_name: str,
    request_file: str,
    client: MLClient,
) -> None:
    endpoint = load_online_endpoint(endpoint_mir_yaml)
    endpoint.name = mir_endpoint_name
    client.online_endpoints.begin_create_or_update(endpoint=endpoint, local=True)

    get_obj = client.online_endpoints.get(name=mir_endpoint_name, local=True)
    assert get_obj is not None

    list_obj = client.online_endpoints.list(local=True)
    assert list_obj is not None
    assert len(list(list_obj)) > 0

    response = client.online_endpoints.invoke(endpoint_name=mir_endpoint_name, request_file=request_file, local=True)
    assert "az ml online-deployment" in response

    client.online_endpoints.begin_delete(name=mir_endpoint_name, local=True)


@pytest.mark.e2etest
@pytest.mark.local_endpoint_local_assets
@pytest.mark.skip()
def test_local_deployment_mir_e2e(
    deployment_create_yaml: str,
    deployment_update_file: str,
    mir_endpoint_name: str,
    request_file: str,
    client: MLClient,
) -> None:
    run_local_endpoint_tests_e2e_create(
        deployment_yaml=deployment_create_yaml,
        update_file=deployment_update_file,
        deployment_name="dep",
        endpoint_name=mir_endpoint_name,
        request_file=request_file,
        client=client,
    )


@pytest.mark.e2etest
@pytest.mark.local_endpoint_local_assets
@pytest.mark.skip()
def test_local_deployment_mir_model_code_overlap_e2e(
    mir_endpoint_name: str,
    request_file: str,
    client: MLClient,
) -> None:
    run_local_endpoint_tests_e2e_create(
        deployment_yaml="tests/test_configs/deployments/online/online_deployment_model_code_overlap.yaml",
        update_file=None,
        deployment_name="dep",
        endpoint_name=mir_endpoint_name,
        request_file=request_file,
        client=client,
    )


@pytest.mark.e2etest
@pytest.mark.local_endpoint_byoc
@pytest.mark.local_endpoint_local_assets
@pytest.mark.skip()
def test_local_deployment_mir_e2e_byoc(
    mir_endpoint_name: str,
    client: MLClient,
) -> None:
    run_local_endpoint_tests_e2e_create(
        deployment_yaml="tests/test_configs/deployments/byoc/tfserving/online_deployment_tfserving.yaml",
        update_file=None,
        deployment_name="dep",
        endpoint_name=mir_endpoint_name,
        request_file="tests/test_configs/deployments/byoc/tfserving/sample_request.json",
        client=client,
        is_sklearn=False,
    )


@pytest.mark.e2etest
@pytest.mark.local_endpoint_byoc
@pytest.mark.skip("Requires building docker image and specific ACR info we don't have from pipelines.")
def test_local_deployment_mir_e2e_byoc_sklearn(
    mir_endpoint_name: str,
    request_file: str,
    client: MLClient,
) -> None:
    ## Uncomment for helping with running BYOC for sklearn scenario
    # register_env(
    #     name="sklearn-env",
    #     version="1",
    #     image="mcr.microsoft.com/azureml/openmpi4.1.0-ubuntu22.04",
    #     conda_file="tests/test_configs/deployments/model-1/environment/conda.yml",
    #     client=client,
    # )
    # workspace = client.workspaces.get(name=client.workspace_name)
    # acr_name = workspace.container_registry.split("/")[-1]
    # # Get acr_image_name from AML Studio for environment after built
    # acr_image_name = "azureml/azureml_0d501feced865646087d1b823b8c3c35"
    # image_name = f"{acr_name}.azurecr.io/{acr_image_name}"
    ## Update deployment yaml with image name
    run_local_endpoint_tests_e2e_create(
        deployment_yaml="tests/test_configs/deployments/byoc/sklearn/online_deployment_byoc_sklearn.yaml",
        update_file=None,
        deployment_name="dep",
        endpoint_name=mir_endpoint_name,
        request_file=request_file,
        client=client,
        is_sklearn=True,
    )


@pytest.mark.e2etest
@pytest.mark.local_endpoint_registered_assets
@pytest.mark.parametrize(
    """deployment_create_yaml,model_to_register,code_to_register,conda_env_to_register""",
    [
        pytest.param(
            "tests/test_configs/deployments/online/online_deployment_registered_model.yaml",
            "tests/test_configs/deployments/model-1/model/sklearn_regression_model.pkl",
            None,
            None,
            marks=pytest.mark.skip(reason="Registered model covered in full registered assets test."),
        ),
        pytest.param(
            "tests/test_configs/deployments/online/online_deployment_registered_code.yaml",
            None,
            "tests/test_configs/deployments/model-1/onlinescoring/",
            None,
            marks=pytest.mark.skip(reason="Registered code covered in full registered assets test."),
        ),
        pytest.param(
            "tests/test_configs/deployments/online/online_deployment_registered_env.yaml",
            None,
            None,
            "tests/test_configs/deployments/model-1/environment/conda.yml",
            marks=pytest.mark.skip(reason="Registered env covered in full registered assets test."),
        ),
        pytest.param(
            "tests/test_configs/deployments/online/online_deployment_registered_artifacts.yaml",
            "tests/test_configs/deployments/model-1/model/sklearn_regression_model.pkl",
            "tests/test_configs/deployments/model-1/onlinescoring/",
            "tests/test_configs/deployments/model-1/environment/conda.yml",
        ),
    ],
)
@pytest.mark.skip()
def test_local_deployment_mir_e2e_registered_artifacts(
    mir_endpoint_name: str,
    request_file: str,
    client: MLClient,
    deployment_create_yaml: str,
    model_to_register: str,
    code_to_register: str,
    conda_env_to_register: str,
) -> None:
    if model_to_register:
        register_model(
            name="sklearn-model",
            version="1",
            path=model_to_register,
            client=client,
        )
    if code_to_register:
        register_code(
            name="code-1",
            version="1",
            path=code_to_register,
            client=client,
        )
    if conda_env_to_register:
        register_env(
            name="sklearn-env",
            version="1",
            image="mcr.microsoft.com/azureml/openmpi4.1.0-ubuntu22.04",
            conda_file=conda_env_to_register,
            client=client,
        )
    run_local_endpoint_tests_e2e_create(
        deployment_yaml=deployment_create_yaml,
        update_file=None,
        deployment_name="dep",
        endpoint_name=mir_endpoint_name,
        request_file=request_file,
        client=client,
    )


def register_model(
    name: str,
    version: str,
    path: str,
    client: MLClient,
):
    model = Model(name=name, path=path, version=version)
    client.models.create_or_update(model)


def register_code(
    name: str,
    version: str,
    path: str,
    client: MLClient,
):
    code = Code(name=name, path=path, version=version)
    client._code.create_or_update(code)


def register_env(
    name: str,
    version: str,
    image: str,
    conda_file: str,
    client: MLClient,
) -> Environment:
    try:
        env = client.environments.get(name=name, version=version)
    except ResourceNotFoundError:
        env = Environment(name=name, image=image, conda_file=conda_file, version=version)
        env = client.environments.create_or_update(env)
    return env


def run_local_endpoint_tests_e2e_create(
    deployment_yaml: str,
    update_file: str,
    endpoint_name: str,
    deployment_name: str,
    request_file: str,
    client: MLClient,
    is_sklearn: bool = True,
) -> None:
    print(f"Creating endpoint with name {endpoint_name}")
    try:
        deployment = load_online_deployment(deployment_yaml)
        deployment.endpoint_name = endpoint_name
        deployment.name = deployment_name
        client.online_deployments.begin_create_or_update(deployment=deployment, no_wait=False, local=True)

        get_obj = client.online_deployments.get(endpoint_name=endpoint_name, name=deployment_name, local=True)
        assert get_obj.name == deployment_name
        assert get_obj.endpoint_name == endpoint_name

        get_obj = client.online_endpoints.get(name=endpoint_name, local=True)
        assert get_obj.name == endpoint_name
        assert get_obj.scoring_uri is not None
        assert get_obj.scoring_uri != ""

        data = client.online_endpoints.invoke(endpoint_name=endpoint_name, request_file=request_file, local=True)
        assert type(data) is str
        if is_sklearn:
            assert "5215" in data
            assert "3726" in data

        logs = client.online_deployments.get_logs(
            endpoint_name=endpoint_name, name=deployment_name, lines=10, local=True
        )
        assert logs
        if is_sklearn:
            assert logs.count("\n") == 10

        deployments = client.online_deployments.list(endpoint_name=endpoint_name, local=True)
        assert deployments is not None
        assert len(list(deployments)) > 0

        if update_file:
            deployment = load_online_deployment(update_file)
            deployment.endpoint_name = endpoint_name
            deployment.name = deployment_name
            client.online_deployments.begin_create_or_update(deployment=deployment, no_wait=False, local=True)

        client.online_deployments.delete(name=deployment_name, endpoint_name=endpoint_name, local=True)
        deployments = client.online_deployments.list(endpoint_name=endpoint_name, local=True)
        assert deployments is not None
        assert endpoint_name not in [dep.endpoint_name for dep in deployments]
    except Exception as e:
        raise e
    finally:
        try:
            client.online_endpoints.begin_delete(name=endpoint_name, local=True)
        except Exception:
            pass



from typing import List, Union

import pytest
import yaml
from msrest import Serializer

from azure.ai.ml._restclient.v2023_04_01_preview.models import DataFactory
from test_utilities.utils import verify_entity_load_and_dump

from azure.ai.ml import load_compute
from azure.ai.ml._restclient.v2023_08_01_preview.models import ComputeResource, ImageMetadata
from azure.ai.ml.constants._compute import CustomApplicationDefaults
from azure.ai.ml.entities import (
    AmlCompute,
    Compute,
    ComputeInstance,
    KubernetesCompute,
    ManagedIdentityConfiguration,
    SynapseSparkCompute,
    VirtualMachineCompute,
    UnsupportedCompute,
)


@pytest.mark.unittest
@pytest.mark.core_sdk_test
class TestComputeEntity:
    def test_compute_from_rest(self):
        with open("tests/test_configs/compute/compute-kubernetes.yaml", "r") as f:
            data = yaml.safe_load(f)
        resource_id = "/subscriptions/dummy/resourceGroups/dummy/providers/Microsoft.Kubernetes/connectedClusters/dummy"
        uai_resource_id = (
            "/subscriptions/dummy/resourceGroups/dummy/providers/Microsoft.ManagedIdentity/userAssignedIdentities/dummy"
        )
        context = {
            "base_path": "./",
            "params_override": [
                {
                    "resource_id": resource_id,
                    "name": "dummy",
                    "namespace": "default",
                    "identity": {
                        "type": "user_assigned",
                        "user_assigned_identities": [{"resource_id": uai_resource_id}],
                    },
                }
            ],
        }
        compute = KubernetesCompute._load_from_dict(data, context)
        compute._to_rest_object()
        assert compute.type == "kubernetes"
        assert compute.identity.type == "user_assigned"
        assert compute.identity.user_assigned_identities[0].resource_id == uai_resource_id

    def _test_loaded_compute(self, compute: AmlCompute):
        assert compute.name == "banchaml"
        assert compute.type == "amlcompute"
        assert compute.location == "eastus"
        assert compute.description == "some_desc_aml"

    def test_compute_from_yaml(self):
        compute: AmlCompute = verify_entity_load_and_dump(
            load_compute,
            self._test_loaded_compute,
            "tests/test_configs/compute/compute-aml.yaml",
        )[0]
        assert compute.ssh_settings.admin_username == "azureuser"
        assert compute.identity.type == "user_assigned"

        rest_intermediate = compute._to_rest_object()
        assert rest_intermediate.properties.compute_type == "AmlCompute"
        assert rest_intermediate.properties.properties.user_account_credentials.admin_user_name == "azureuser"
        assert rest_intermediate.properties.properties.enable_node_public_ip
        assert rest_intermediate.properties.disable_local_auth is False
        assert rest_intermediate.location == compute.location
        assert rest_intermediate.tags is not None
        assert rest_intermediate.tags["test"] == "true"
        assert rest_intermediate.properties.disable_local_auth is False
        assert rest_intermediate.properties.properties.remote_login_port_public_access == "Enabled"

        serializer = Serializer({"ComputeResource": ComputeResource})
        body = serializer.body(rest_intermediate, "ComputeResource")
        assert body["identity"]["type"] == "UserAssigned"
        assert body["identity"]["userAssignedIdentities"] == self._uai_list_to_dict(
            compute.identity.user_assigned_identities
        )
        assert body["location"] == compute.location

    def test_aml_compute_from_yaml_with_disable_public_access(self):

        compute: AmlCompute = verify_entity_load_and_dump(
            load_compute,
            self._test_loaded_compute,
            "tests/test_configs/compute/compute-aml-disable-public-access.yaml",
        )[0]

        rest_intermediate = compute._to_rest_object()

        assert rest_intermediate.properties.compute_type == "AmlCompute"
        assert rest_intermediate.properties.properties.enable_node_public_ip
        assert rest_intermediate.properties.disable_local_auth is True
        assert rest_intermediate.location == compute.location
        assert rest_intermediate.properties.properties.remote_login_port_public_access == "NotSpecified"

    def test_aml_compute_from_yaml_with_creds_and_disable_public_access(self):
        compute: AmlCompute = load_compute("tests/test_configs/compute/compute-aml-no-identity.yaml")
        compute.ssh_public_access_enabled = False

        rest_intermediate = compute._to_rest_object()

        assert rest_intermediate.properties.compute_type == "AmlCompute"
        assert rest_intermediate.properties.properties.enable_node_public_ip
        assert rest_intermediate.properties.disable_local_auth is False
        assert rest_intermediate.location == compute.location
        assert rest_intermediate.properties.properties.remote_login_port_public_access == "Disabled"

    def test_aml_compute_from_yaml_with_disable_public_access_when_no_sshSettings(self):

        compute: AmlCompute = verify_entity_load_and_dump(
            load_compute,
            self._test_loaded_compute,
            "tests/test_configs/compute/compute-aml-public-access-no-ssh.yaml",
        )[0]

        rest_intermediate = compute._to_rest_object()

        assert rest_intermediate.properties.compute_type == "AmlCompute"
        assert rest_intermediate.properties.properties.enable_node_public_ip
        assert rest_intermediate.properties.disable_local_auth is True
        assert rest_intermediate.properties.properties.remote_login_port_public_access == "Enabled"
        assert rest_intermediate.location == compute.location

    def test_compute_vm_from_yaml(self):
        resource_id = "/subscriptions/13e50845-67bc-4ac5-94db-48d493a6d9e8/resourceGroups/myrg/providers/Microsoft.Compute/virtualMachines/myvm"
        fake_key = "myfakekey"
        compute: VirtualMachineCompute = load_compute("tests/test_configs/compute/compute-vm.yaml")
        assert compute.name == "banchcivm"
        assert compute.ssh_settings.admin_username == "azureuser"
        assert compute.ssh_settings.admin_password == "azureuserpassword"
        assert compute.ssh_settings.ssh_port == 8888
        assert compute.resource_id == resource_id
        assert compute.ssh_settings.ssh_private_key_file == "tests/test_configs/compute/ssh_fake_key.txt"

        rest_intermediate = compute._to_rest_object()
        assert rest_intermediate.properties.compute_type == "VirtualMachine"
        assert rest_intermediate.properties.resource_id == resource_id
        assert rest_intermediate.properties.properties.ssh_port == 8888
        assert rest_intermediate.properties.properties.administrator_account.password == "azureuserpassword"
        assert rest_intermediate.properties.properties.administrator_account.username == "azureuser"
        assert rest_intermediate.properties.properties.administrator_account.private_key_data == fake_key

        serializer = Serializer({"ComputeResource": ComputeResource})
        body = serializer.body(rest_intermediate, "ComputeResource")
        assert body["properties"]["resourceId"] == resource_id
        assert body["properties"]["properties"]["sshPort"] == 8888
        assert body["properties"]["properties"]["administratorAccount"]["username"] == "azureuser"
        assert body["properties"]["properties"]["administratorAccount"]["password"] == "azureuserpassword"
        assert body["properties"]["properties"]["administratorAccount"]["privateKeyData"] == fake_key

    def test_compute_from_constructor(self):
        compute = ComputeInstance(name="comp", type="computeinstance")
        assert compute.type == "computeinstance"

        compute = KubernetesCompute(name="mykube", namespace="default", properties={})
        compute._to_dict()
        assert compute.type == "kubernetes"

    def _uai_list_to_dict(self, value: List[ManagedIdentityConfiguration]) -> Union[str, ManagedIdentityConfiguration]:
        uai_dict = {}

        for item in value:
            resource_id = item.resource_id
            uai_dict[resource_id] = {}
        return uai_dict

    def test_compute_instance_load_from_rest(self):
        compute_instance: ComputeInstance = load_compute(
            source="tests/test_configs/compute/compute-ci-unit.yaml",
            params_override=[{"tags.test1": "test"}, {"tags.test2": "true"}, {"tags.test3": "0"}],
        )
        compute_instance._set_full_subnet_name("subscription_id", "resource_group_name")
        assert compute_instance.ssh_settings is not None
        assert compute_instance.ssh_settings.ssh_key_value.startswith("ssh-rsa")
        compute_resource = compute_instance._to_rest_object()

        compute_instance2: ComputeInstance = ComputeInstance._load_from_rest(compute_resource)
        assert compute_instance2.last_operation == compute_instance.last_operation
        assert compute_instance2.services == compute_instance.services
        assert compute_instance2.ssh_settings is not None
        assert compute_instance2.ssh_settings.ssh_key_value.startswith("ssh-rsa")
        assert compute_instance2.tags is not None
        assert compute_instance2.tags["test1"] == "test"
        assert compute_instance2.tags["test2"] == "true"
        assert compute_instance2.tags["test3"] == "0"
        assert compute_instance2.enable_sso is False
        assert compute_instance2.enable_root_access is False
        assert compute_instance2.enable_os_patching is True
        assert compute_instance2.release_quota_on_stop is True

        compute_instance3: ComputeInstance = load_compute(
            source="tests/test_configs/compute/compute-ci-defaults-unit.yaml",
        )._to_rest_object()
        assert compute_instance3.properties.compute_type == "ComputeInstance"
        assert compute_instance3.properties.properties.enable_sso is True
        assert compute_instance3.properties.properties.enable_root_access is True
        assert compute_instance3.properties.properties.enable_os_patching is False
        assert compute_instance3.properties.properties.release_quota_on_stop is False

    def test_compute_instance_with_image_metadata(self):
        os_image_metadata = ImageMetadata(
            current_image_version="22.08.19",
            latest_image_version="22.08.20",
            is_latest_os_image_version=False,
        )
        compute_instance: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-unit.yaml")
        compute_resource = compute_instance._to_rest_object()
        compute_resource.properties.properties.os_image_metadata = os_image_metadata
        compute_instance2: ComputeInstance = ComputeInstance._load_from_rest(compute_resource)
        os_image_metadata2 = compute_instance2.os_image_metadata
        assert os_image_metadata2 is not None
        assert os_image_metadata2.current_image_version == os_image_metadata.current_image_version
        assert os_image_metadata2.latest_image_version == os_image_metadata.latest_image_version
        assert os_image_metadata2.is_latest_os_image_version == os_image_metadata.is_latest_os_image_version

    def test_compute_instance_schedules_from_yaml(self):
        compute_instance: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-schedules.yaml")
        assert len(compute_instance.schedules.compute_start_stop) == 2

        compute_resource = compute_instance._to_rest_object()
        compute_instance2: ComputeInstance = ComputeInstance._load_from_rest(compute_resource)
        assert len(compute_instance2.schedules.compute_start_stop) == 2
        assert compute_instance2.schedules.compute_start_stop[0].action == "stop"
        assert compute_instance2.schedules.compute_start_stop[0].trigger.type == "Cron"
        assert compute_instance2.schedules.compute_start_stop[0].trigger.start_time == "2021-03-10T21:21:07"
        assert compute_instance2.schedules.compute_start_stop[0].trigger.time_zone == "Pacific Standard Time"
        assert compute_instance2.schedules.compute_start_stop[0].trigger.expression == "0 18 * * *"
        assert compute_instance2.schedules.compute_start_stop[1].action == "start"
        assert compute_instance2.schedules.compute_start_stop[1].trigger.type == "Recurrence"
        assert compute_instance2.schedules.compute_start_stop[1].trigger.start_time == "2021-03-10T21:21:07"
        assert compute_instance2.schedules.compute_start_stop[1].trigger.time_zone == "Pacific Standard Time"
        assert compute_instance2.schedules.compute_start_stop[1].trigger.frequency == "week"
        assert compute_instance2.schedules.compute_start_stop[1].trigger.interval == 1
        assert compute_instance2.schedules.compute_start_stop[1].trigger.schedule is not None

    def test_compute_instance_idle_shutdown_from_yaml(self):
        compute_instance: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-idle-shutdown.yaml")
        assert compute_instance.idle_time_before_shutdown == "PT20M"
        assert compute_instance.idle_time_before_shutdown_minutes == 15

        compute_resource = compute_instance._to_rest_object()
        assert (
            compute_resource.properties.properties.idle_time_before_shutdown
            == f"PT{compute_instance.idle_time_before_shutdown_minutes}M"
        )

        compute_instance2: ComputeInstance = ComputeInstance._load_from_rest(compute_resource)
        assert compute_instance2.idle_time_before_shutdown == f"PT{compute_instance.idle_time_before_shutdown_minutes}M"
        assert compute_instance2.idle_time_before_shutdown_minutes == compute_instance.idle_time_before_shutdown_minutes

    def test_compute_instance_setup_scripts_from_yaml(self):
        loaded_instance: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-setup-scripts.yaml")
        compute_resource: ComputeResource = loaded_instance._to_rest_object()
        compute_instance: ComputeInstance = ComputeInstance._load_from_rest(compute_resource)

        assert compute_instance.setup_scripts is not None
        assert compute_instance.setup_scripts.creation_script is not None
        assert compute_instance.setup_scripts.creation_script.path == "Users/test/creation-script.sh"
        assert compute_instance.setup_scripts.creation_script.timeout_minutes == "20"
        assert compute_instance.setup_scripts.startup_script is not None
        assert compute_instance.setup_scripts.startup_script.path == "Users/test/startup-script.sh"
        assert compute_instance.setup_scripts.startup_script.command == "ls"
        assert compute_instance.setup_scripts.startup_script.timeout_minutes == "15"

    def test_compute_instance_uai_from_yaml(self):
        compute: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-uai.yaml")
        assert compute.name == "banchci"
        assert compute.type == "computeinstance"
        assert compute.identity.type == "user_assigned"
        assert compute.identity.user_assigned_identities
        assert len(compute.identity.user_assigned_identities) == 1
        assert (
            compute.identity.user_assigned_identities[0].resource_id
            == "/subscriptions/4faaaf21-663f-4391-96fd-47197c630979/resourceGroups/test-rg-centraluseuap-v2-t-2021W35"
            "/providers/Microsoft.ManagedIdentity/userAssignedIdentities/x"
        )
        assert compute.ssh_public_access_enabled == False

        compute_resource = compute._to_rest_object()
        assert compute_resource.identity.type == "UserAssigned"
        assert len(compute_resource.identity.user_assigned_identities) == 1
        for k in compute_resource.identity.user_assigned_identities.keys():
            assert (
                k == "/subscriptions/4faaaf21-663f-4391-96fd-47197c630979/resourceGroups/test-rg"
                "-centraluseuap-v2-t-2021W35/providers/Microsoft.ManagedIdentity"
                "/userAssignedIdentities/x"
            )
        assert compute_resource.properties.properties.ssh_settings.ssh_public_access == "Disabled"
        assert compute_resource.properties.properties.ssh_settings.admin_public_key is None

        compute_from_rest = Compute._from_rest_object(compute_resource)
        assert compute_from_rest.type == "computeinstance"
        assert compute_from_rest.identity.type == "user_assigned"
        assert compute_from_rest.identity.user_assigned_identities
        assert len(compute_from_rest.identity.user_assigned_identities) == 1
        assert (
            compute_from_rest.identity.user_assigned_identities[0].resource_id
            == "/subscriptions/4faaaf21-663f-4391-96fd-47197c630979/resourceGroups/test-rg-centraluseuap-v2-t-2021W35"
            "/providers/Microsoft.ManagedIdentity/userAssignedIdentities/x"
        )
        assert compute_from_rest.ssh_public_access_enabled == False

    def test_compute_instace_to_rest(self):
        compute: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci.yaml")
        compute.ssh_public_access_enabled = True

        compute_rest = compute._to_rest_object()

        assert compute_rest.location == compute.location
        assert compute_rest.properties.properties.ssh_settings.ssh_public_access == "Enabled"
        assert compute_rest.properties.properties.ssh_settings.admin_public_key == compute.ssh_settings.ssh_key_value
        assert compute_rest.properties.description == compute.description
        assert compute_rest.properties.compute_type == "ComputeInstance"
        assert compute_rest.properties.disable_local_auth == False

    def test_compute_instance_sai_from_yaml(self):
        compute: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-id-sys-assigned.yaml")
        assert compute.name == "banchci"
        assert compute.type == "computeinstance"
        assert compute.identity.type == "system_assigned"

        compute_resource = compute._to_rest_object()
        assert compute_resource.identity.type == "SystemAssigned"
        assert compute_resource.properties.disable_local_auth == True

        compute_from_rest = Compute._from_rest_object(compute_resource)
        assert compute_from_rest.type == "computeinstance"
        assert compute_from_rest.identity.type == "system_assigned"

    def test_compute_no_public_ip_from_yaml(self):
        compute_instance: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-no-public-ip.yaml")

        aml_compute: AmlCompute = load_compute("tests/test_configs/compute/compute-aml-no-public-ip.yaml")

        def validate_no_public_ip(compute: Compute):
            assert compute.enable_node_public_ip == False
            compute_resource = compute._to_rest_object()
            assert compute_resource.properties.properties.enable_node_public_ip == False
            compute_from_rest = Compute._from_rest_object(compute_resource)
            assert compute_from_rest.enable_node_public_ip == False

        validate_no_public_ip(compute=compute_instance)
        validate_no_public_ip(compute=aml_compute)

    def test_compute_instance_with_custom_app(self):
        compute_instance: ComputeInstance = load_compute("tests/test_configs/compute/compute-ci-custom-app.yaml")
        assert compute_instance.custom_applications is not None
        assert len(compute_instance.custom_applications) == 2

        custom_app = compute_instance.custom_applications[0]
        assert custom_app.name == "rstudio-workbench"
        assert custom_app.type == CustomApplicationDefaults.DOCKER
        assert custom_app.image is not None
        assert custom_app.image.reference == "ghcr.io/azure/rstudio-workbench:latest"
        assert custom_app.endpoints is not None
        assert custom_app.endpoints[0].target == 8787
        assert custom_app.endpoints[0].published == 8788
        assert custom_app.environment_variables is not None
        assert custom_app.environment_variables["RSP_LICENSE"] == "XXXX-XXXX-XXXX-XXXX-XXXX-XXXX-XXXX"
        assert custom_app.bind_mounts is not None
        assert custom_app.bind_mounts[0].target == "/home/azureuser"
        assert custom_app.bind_mounts[0].source == "/home/azureuser"

        custom_app = compute_instance.custom_applications[1]
        assert custom_app.name == "rstudio-workbench2"
        assert custom_app.endpoints[0].published == 8789
        assert custom_app.environment_variables["RSP_LICENSE"] == "XXXX-XXXX-XXXX-XXXX-XXXX-XXXX-YYYY"

        compute_rest_obj = compute_instance._to_rest_object()
        compute_instance2 = ComputeInstance._from_rest_object(compute_rest_obj)
        assert len(compute_instance2.custom_applications) == 2

        custom_app3 = compute_instance2.custom_applications[0]
        assert custom_app3.name == "rstudio-workbench"
        assert custom_app3.image is not None
        assert custom_app3.image.reference == "ghcr.io/azure/rstudio-workbench:latest"
        assert custom_app3.endpoints is not None
        assert custom_app3.endpoints[0].target == 8787
        assert custom_app3.endpoints[0].published == 8788
        assert custom_app3.environment_variables is not None
        assert custom_app3.environment_variables["RSP_LICENSE"] == "XXXX-XXXX-XXXX-XXXX-XXXX-XXXX-XXXX"
        assert custom_app3.bind_mounts is not None
        assert custom_app3.bind_mounts[0].target == "/home/azureuser"
        assert custom_app3.bind_mounts[0].source == "/home/azureuser"

        custom_app4 = compute_instance2.custom_applications[1]
        assert custom_app4.name == "rstudio-workbench2"
        assert custom_app4.endpoints[0].published == 8789
        assert custom_app4.environment_variables["RSP_LICENSE"] == "XXXX-XXXX-XXXX-XXXX-XXXX-XXXX-YYYY"

    def test_synapse_compute_from_rest(self):
        with open("tests/test_configs/compute/compute-synapsespark.yaml", "r") as f:
            data = yaml.safe_load(f)
        resource_id = "/subscriptions/dummy/resourceGroups/dummy/providers/Microsoft.Synapse/workspaces/dummy/bigDataPools/dummypool"
        context = {
            "base_path": "./",
            "params_override": [{"resource_id": resource_id, "name": "dummy"}],
        }
        compute = SynapseSparkCompute._load_from_dict(data, context)
        compute._to_rest_object()
        assert compute.type == "synapsespark"

    def test_synapsespark_compute_from_yaml(self):
        compute: SynapseSparkCompute = load_compute("tests/test_configs/compute/compute-synapsespark-identity.yaml")
        assert compute.name == "testidentity"
        assert compute.identity.type == "user_assigned"

        rest_intermediate = compute._to_rest_object()
        assert rest_intermediate.properties.compute_type == "SynapseSpark"
        serializer = Serializer({"ComputeResource": ComputeResource})
        body = serializer.body(rest_intermediate, "ComputeResource")
        assert body["identity"]["type"] == "UserAssigned"
        assert body["identity"]["userAssignedIdentities"] == self._uai_list_to_dict(
            compute.identity.user_assigned_identities
        )

    def test_deserialize_data_factory_compute(self):
        rest_compute = ComputeResource(properties=DataFactory())
        # DataFactory compute can load from rest object
        UnsupportedCompute._load_from_rest(rest_compute)



import importlib
import re
import sys
from importlib import reload

import pip
import pytest

from azure.ai.ml._utils import _func_utils


class MockClass:
    mock_cls_attr = 2

    def __init__(self, mock_arg):
        self.mock_arg = mock_arg

    def mock_instance_func(self, arg):
        result = self.mock_arg + arg
        return result

    @classmethod
    def mock_class_method(cls, arg):
        result = cls.mock_cls_attr + arg
        return result

    def __call__(self, arg):
        result = self.mock_arg + arg
        return result


def mock_conflict_function(__self):
    return __self


def mock_function_with_self(self):
    return self


is_true = True


def mock_function_multi_return(mock_arg):
    if is_true:
        mock_local_variable = 1
        return mock_local_variable, mock_arg
    else:
        mock_local_variable = 2
        return mock_local_variable, mock_arg


def mock_function_multi_return_expected(__self, mock_arg):
    try:
        if is_true:
            mock_local_variable = 1
            return mock_local_variable, mock_arg
        else:
            mock_local_variable = 2
            return mock_local_variable, mock_arg
    finally:
        __self._locals = locals().copy()


@pytest.mark.unittest
@pytest.mark.pipeline_test
class TestPersistentLocalsProfiler:
    @classmethod
    def get_persistent_locals_builder(cls) -> _func_utils.PersistentLocalsFunctionBuilder:
        return _func_utils.PersistentLocalsFunctionProfilerBuilder()

    def get_outputs_and_locals(self, func, injected_params):
        return self.get_persistent_locals_builder().call(func, injected_params)

    def test_simple(self):
        def mock_function(mock_arg):
            mock_local_variable = 1
            return mock_local_variable, mock_arg

        outputs, _locals = self.get_outputs_and_locals(mock_function, {"mock_arg": 1})
        assert outputs == (1, 1)
        assert set(_locals.keys()) == {"mock_arg", "mock_local_variable"}

    def test_func_with_named_self_argument(self):
        outputs, _locals = self.get_outputs_and_locals(mock_function_with_self, {"self": 1})
        assert outputs == 1
        assert set(_locals.keys()) == {"self"}

    def test_raise_exception(self):
        def mock_error_exception():
            mock_local_variable = 1
            return mock_local_variable / 0

        with pytest.raises(ZeroDivisionError):
            self.get_outputs_and_locals(mock_error_exception, {})

    def test_instance_func(self):
        mock_obj = MockClass(1)
        outputs, _locals = self.get_outputs_and_locals(mock_obj.mock_instance_func, {"arg": 1})
        assert outputs == 2
        assert set(_locals.keys()) == {"result", "arg", "self"}

    def test_class_method(self):
        mock_obj = MockClass(1)
        outputs, _locals = self.get_outputs_and_locals(mock_obj.mock_class_method, {"arg": 1})
        assert outputs == 3
        assert set(_locals.keys()) == {"result", "arg", "cls"}

    def test_instance_call(self):
        mock_obj = MockClass(1)
        outputs, _locals = self.get_outputs_and_locals(mock_obj, {"arg": 1})
        assert outputs == 2
        assert set(_locals.keys()) == {"result", "arg", "self"}

    def test_invalid_passed_func(self):
        with pytest.raises(TypeError, match="func must be a function or a callable object"):
            self.get_outputs_and_locals(1, {"arg": 1})

    def test_param_conflict(self):
        with pytest.raises(
            ValueError,
            match=re.escape("Injected param name __self conflicts with function args ['__self']"),
        ):
            self.get_outputs_and_locals(mock_conflict_function, {"arg": 1})

    def test_multiple_return(self):
        outputs, _locals = self.get_outputs_and_locals(mock_function_multi_return, {"mock_arg": 1})
        assert outputs == (1, 1)
        assert set(_locals.keys()) == {"mock_arg", "mock_local_variable"}


@pytest.mark.unittest
@pytest.mark.pipeline_test
@pytest.mark.usefixtures("enable_pipeline_private_preview_features")
class TestPersistentLocalsPrivatePreview(TestPersistentLocalsProfiler):
    _PACKAGE = "bytecode"

    @classmethod
    def setup_class(cls):
        """setup any state specific to the execution of the given class (which
        usually contains tests).
        """
        try:
            importlib.import_module(cls._PACKAGE)
        except ImportError:
            pip.main(["install", cls._PACKAGE])
            globals()[cls._PACKAGE] = importlib.import_module(cls._PACKAGE)

            # reload persistent locals function builder after installing bytecode
            reload(_func_utils)

    @classmethod
    def teardown_class(cls):
        """teardown any state that was previously setup with a call to
        setup_class.
        """
        if cls._PACKAGE not in globals():
            return

        del globals()[cls._PACKAGE]
        pip.main(["uninstall", cls._PACKAGE])
        reload(_func_utils)

    @classmethod
    def instr_to_str(cls, instr, labels):
        from bytecode import Label

        def _label_to_str(_label):
            if id(_label) not in labels:
                labels.append(id(_label))
            return f"{_label.__class__.__name__} {labels.index(id(_label))}"

        if isinstance(instr, Label):
            return _label_to_str(instr)

        if isinstance(instr.arg, Label):
            return f"{instr.name} {_label_to_str(instr.arg)}"

        return f"{instr.name} {instr.arg}"

    @classmethod
    def instructions_to_str(cls, instructions):
        labels = []
        return [cls.instr_to_str(instr, labels) for instr in instructions]

    @classmethod
    def get_persistent_locals_builder(cls):
        return _func_utils.PersistentLocalsFunctionBytecodeBuilder()

    def test_multiple_return(self):
        if (3, 8) > sys.version_info:
            super().test_multiple_return()
        else:
            outputs, _locals = self.get_outputs_and_locals(mock_function_multi_return, {"mock_arg": 1})
            assert outputs == (1, 1)

            # TODO: persist locals for multiple return in Python 3.8/3.9/3.10/3.11
            # builder = self.get_persistent_locals_builder()
            # expected_instructions = builder.get_instructions(mock_function_multi_return_expected)
            # actual_instructions = builder._build_instructions(mock_function_multi_return)
            # assert self.instructions_to_str(actual_instructions) == self.instructions_to_str(expected_instructions)


@pytest.mark.unittest
@pytest.mark.pipeline_test
@pytest.mark.skipif(
    condition=sys.version_info >= (3, 11), reason="historical implementation doesn't support Python 3.11+"
)
class TestPersistentLocalsHistoricalImplementation(TestPersistentLocalsPrivatePreview):
    """This is to test the implementation of persistent locals function in azuerml-components."""

    @classmethod
    def get_persistent_locals_builder(cls):
        from bytecode import Instr, Label

        class HistoricalPersistentLocalsFunctionProfilerBuilder(_func_utils.PersistentLocalsFunctionBytecodeBuilder):
            def _build_instructions(self, func):
                bytecode = self.get_instructions(func)
                # Add `try` at the begining of the code
                finally_label = Label()
                bytecode.insert(0, Instr("SETUP_FINALLY", finally_label))
                # Add `final` at the end of the code
                added_param = "__self"

                copy_locals_instructions = [
                    # __self._locals = locals().copy()
                    Instr("LOAD_GLOBAL", "locals"),
                    Instr("CALL_FUNCTION", 0),
                    Instr("LOAD_ATTR", "copy"),
                    Instr("CALL_FUNCTION", 0),
                    Instr("LOAD_FAST", added_param),
                    Instr("STORE_ATTR", "_locals"),
                ]

                if sys.version_info < (3, 8):
                    # python 3.6 and 3.7
                    bytecode.extend(
                        [finally_label]
                        + copy_locals_instructions
                        + [Instr("END_FINALLY"), Instr("LOAD_CONST", None), Instr("RETURN_VALUE")]
                    )
                elif sys.version_info < (3, 9):
                    # In python 3.8, add new instruction CALL_FINALLY
                    # https://docs.python.org/3.8/library/dis.html?highlight=call_finally#opcode-CALL_FINALLY
                    bytecode.insert(-1, Instr("CALL_FINALLY", finally_label))
                    bytecode.extend(
                        [finally_label]
                        + copy_locals_instructions
                        + [Instr("END_FINALLY"), Instr("LOAD_CONST", None), Instr("RETURN_VALUE")]
                    )
                elif sys.version_info < (3, 10):
                    # In python 3.9, add new instruction RERAISE and CALL_FINALLY is removed.
                    # https://docs.python.org/3.9/library/dis.html#opcode-RERAISE
                    raise_error = Label()
                    extend_instructions = (
                        copy_locals_instructions
                        + [Instr("JUMP_FORWARD", raise_error), finally_label]
                        + copy_locals_instructions
                        + [Instr("RERAISE"), raise_error]
                    )
                    bytecode[-1:-1] = extend_instructions
                else:
                    # python 3.10 and above
                    bytecode[-1:-1] = copy_locals_instructions
                    bytecode.extend([finally_label] + copy_locals_instructions + [Instr("RERAISE", 0)])
                return bytecode

        return HistoricalPersistentLocalsFunctionProfilerBuilder()

    def test_multiple_return(self):
        # historical implementation can save locals correctly for multiple return in Python 3.7 only
        if (3, 8) > sys.version_info:
            super().test_multiple_return()
        else:
            outputs, _locals = self.get_outputs_and_locals(mock_function_multi_return, {"mock_arg": 1})
            assert outputs == (1, 1)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

"""Entrypoint for creating Distillation task."""
from typing import Any, Dict, Optional, Union

from azure.ai.ml._utils._experimental import experimental
from azure.ai.ml.constants import DataGenerationType
from azure.ai.ml.constants._common import AssetTypes
from azure.ai.ml.entities._inputs_outputs import Input
from azure.ai.ml.entities._job.distillation.distillation_job import DistillationJob
from azure.ai.ml.entities._job.distillation.prompt_settings import PromptSettings
from azure.ai.ml.entities._job.distillation.teacher_model_settings import TeacherModelSettings
from azure.ai.ml.entities._job.resource_configuration import ResourceConfiguration
from azure.ai.ml.entities._workspace.connections.workspace_connection import WorkspaceConnection


@experimental
def distillation(
    *,
    experiment_name: str,
    data_generation_type: str,
    data_generation_task_type: str,
    teacher_model_endpoint_connection: WorkspaceConnection,
    student_model: Union[Input, str],
    training_data: Optional[Union[Input, str]] = None,
    validation_data: Optional[Union[Input, str]] = None,
    teacher_model_settings: Optional[TeacherModelSettings] = None,
    prompt_settings: Optional[PromptSettings] = None,
    hyperparameters: Optional[Dict] = None,
    resources: Optional[ResourceConfiguration] = None,
    **kwargs: Any,
) -> "DistillationJob":
    """Function to create a Distillation job.

    A distillation job is used to transfer knowledge from a teacher model to student model by a two step process of
    generating synthetic data from the teacher model and then finetuning the student model with the generated
    synthetic data.

    :param experiment_name: The name of the experiment.
    :type experiment_name: str
    :param data_generation_type: The type of data generation to perform.

        Acceptable values: label_generation
    :type data_generation_type: str
    :param data_generation_task_type: The type of data to generate

        Acceptable values: NLI, NLU_QA, CONVERSATION, MATH, SUMMARIZATION
    :type: data_generation_task_type: str
    :param teacher_model_endpoint_connection: The kind of teacher model connection that includes the name, endpoint
        url, and api_key.
    :type: teacher_model_endpoint_connection: WorkspaceConnection
    :param student_model: The model to train
    :type student_model: typing.Union[Input, str]
    :param training_data: The training data to use. Should contain the questions but not the labels, defaults to None
    :type training_data: typing.Optional[typing.Union[Input, str]], optional
    :param validation_data: The validation data to use. Should contain the questions but not the labels, defaults to
        None
    :type validation_data: typing.Optional[typing.Union[Input, str]], optional
    :param teacher_model_settings: The settings for the teacher model. Accepts both the inference parameters and
        endpoint settings, defaults to None

        Acceptable keys for inference parameters: temperature, max_tokens, top_p, frequency_penalty, presence_penalty,
        stop
    :type teacher_model_settings: typing.Optional[TeacherModelSettings], optional
    :param prompt_settings: The settings for the prompt that affect the system prompt used for data generation,
        defaults to None
    :type prompt_settings: typing.Optional[PromptSettings], optional
    :param hyperparameters: The hyperparameters to use for finetuning, defaults to None
    :type hyperparameters: typing.Optional[typing.Dict], optional
    :param resources: The compute resource to use for the data generation step in the distillation job, defaults to
        None
    :type resources: typing.Optional[ResourceConfiguration], optional
    :raises ValueError: Raises ValueError if there is no training data and data generation type is 'label_generation'
    :return: A DistillationJob to submit
    :rtype: DistillationJob
    """
    if isinstance(student_model, str):
        student_model = Input(type=AssetTypes.URI_FILE, path=student_model)
    if isinstance(training_data, str):
        training_data = Input(type=AssetTypes.URI_FILE, path=training_data)
    if isinstance(validation_data, str):
        validation_data = Input(type=AssetTypes.URI_FILE, path=validation_data)

    if training_data is None and data_generation_type == DataGenerationType.LABEL_GENERATION:
        raise ValueError(
            f"Training data can not be None when data generation type is set to "
            f"{DataGenerationType.LABEL_GENERATION}."
        )

    if validation_data is None and data_generation_type == DataGenerationType.LABEL_GENERATION:
        raise ValueError(
            f"Validation data can not be None when data generation type is set to "
            f"{DataGenerationType.LABEL_GENERATION}."
        )

    return DistillationJob(
        data_generation_type=data_generation_type,
        data_generation_task_type=data_generation_task_type,
        teacher_model_endpoint_connection=teacher_model_endpoint_connection,
        student_model=student_model,
        training_data=training_data,
        validation_data=validation_data,
        teacher_model_settings=teacher_model_settings,
        prompt_settings=prompt_settings,
        hyperparameters=hyperparameters,
        resources=resources,
        experiment_name=experiment_name,
        **kwargs,
    )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


import json
from pathlib import Path
from typing import Dict, List, Optional

from azure.ai.ml._local_endpoints.utilities.wsl_utility import get_wsl_path, in_wsl
from azure.ai.ml._local_endpoints.vscode_debug.devcontainer_properties import (
    AppPort,
    Build,
    ContainerEnv,
    Extensions,
    ForwardPorts,
    Image,
    Mounts,
    OverrideCommand,
    RunArgs,
    Settings,
)
from azure.ai.ml.constants._common import DefaultOpenEncoding
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationException


class DevContainerResolver:
    """DevContainerResolver class represents the collection of properties of a devcontainer.json.

    Reference: https://code.visualstudio.com/docs/remote/devcontainerjson-reference
    """

    def __init__(
        self,
        image: Optional[str] = None,
        dockerfile_path: str = "../Dockerfile",
        build_context: Optional[str] = None,
        build_target: Optional[str] = None,
        environment: Optional[dict] = None,
        mounts: Optional[dict] = None,
        labels: Optional[dict] = None,
        port: int = 5001,
    ):
        """Resolves the devcontainer.json based on provided properties.

        :param image: name of local deployment
        :type image: str
        :param dockerfile_path: path to Dockerfile relative to devcontainer.json
        :type dockerfile_path: str
        :param build_context: build directory on user's local system
        :type build_context: str
        :param build_target: directory on user's local system where Dockerfile is located
        :type build_target: str
        :param environment: dictionary of docker environment variables to set in dev container
        :type environment: dict
        :param mounts: dictionary of volumes to mount to dev container
        :type mounts: dict
        :param labels: dictionary of labels to add to dev container
        :type labels: dict
        :param port: Port exposed in Docker image for AzureML service.
        :type port: int
        """
        if not (image or (build_context and dockerfile_path)):
            msg = "Must provide image or build context for devcontainer.json"
            raise ValidationException(
                message=msg,
                no_personal_data_message=msg,
                target=ErrorTarget.LOCAL_ENDPOINT,
                error_category=ErrorCategory.USER_ERROR,
            )
        self._local_path: Optional[str] = None
        self._properties: Optional[dict] = {}

        self._image: Optional[str] = image
        self._dockerfile_path: str = dockerfile_path
        self._build_context: Optional[str] = build_context
        self._build_target: Optional[str] = build_target
        self._environment: Optional[dict] = environment
        self._mounts: list = _reformat_mounts(mounts) if mounts else mounts  # type: ignore[assignment]
        self._labels: list = _reformat_labels(labels) if labels else labels  # type: ignore[assignment]
        self._port = port
        self._construct()

    @property
    def local_path(self) -> Optional[str]:
        """Returns the local path of the devcontainer.json.

        :return: str
        """
        return self._local_path

    def _construct(self) -> None:
        """Constructs the devcontainer properties based on attributes."""
        if self._image:
            self._properties = Image(image=self._image).to_dict()
        elif self._dockerfile_path and self._build_context:
            self._properties = Build(
                dockerfile_path=self._dockerfile_path,
                build_context=self._build_context,
                target=self._build_target,
            ).to_dict()

        if self._properties is not None:
            self._properties.update(OverrideCommand().to_dict())
            self._properties.update(Extensions().to_dict())
            self._properties.update(Settings().to_dict())

            if self._environment:
                self._properties.update(ContainerEnv(environment_variables=self._environment).to_dict())
            if self._mounts:
                self._properties.update(Mounts(mounts=self._mounts).to_dict())
            if self._labels:
                self._properties.update(RunArgs(labels=self._labels).to_dict())
            if self._port:
                self._properties.update(AppPort(port=self._port).to_dict())
                self._properties.update(ForwardPorts(port=self._port).to_dict())

    def write_file(self, directory_path: str) -> None:
        """Writes this devcontainer.json to provided directory.

        :param directory_path: absolute path of local directory to write devcontainer.json.
        :type directory_path: str
        """
        self._local_path = get_wsl_path(directory_path) if in_wsl() else directory_path

        file_path = _get_devcontainer_file_path(directory_path=directory_path)
        with open(file_path, "w", encoding=DefaultOpenEncoding.WRITE) as f:
            f.write(f"{json.dumps(self._properties, indent=4)}\n")


def _reformat_mounts(mounts: Dict[str, Dict[str, Dict[str, str]]]) -> List[str]:
    """Reformat mounts from Docker format to DevContainer format.

    :param mounts: Dictionary with mount information for Docker container. For example:
        .. code-block:: python

            {
                "<unique mount key>": {
                    "<local_source>": {
                        "<mount type i.e. bind>": "<container_dest>"
                    }
                }
            }

    :type mounts: dict
    :return:
       ["source=${localWorkspaceFolder}/app-scripts, target=/usr/local/share/app-scripts,type=bind,consistency=cached"]
    :rtype: List[str]
    """
    devcontainer_mounts = []
    for mount_dict in mounts.values():
        for source, dest in mount_dict.items():
            for mount_type, container_dest in dest.items():
                devcontainer_mounts.append(f"source={source},target={container_dest},type={mount_type}")
    return devcontainer_mounts


def _reformat_labels(labels: Dict[str, str]) -> List[str]:
    """Reformat labels from Docker format to DevContainer format.

    :param labels: Dictionary with label information for Docker container. For example:
        .. code-block:: python

            {
                "key": "value",
                "key1": "value1"
            }

    :type labels: Dict[str, str]
    :return: ["--label=key=value", "--label=key1=value1"]
    :rtype: List[str]
    """
    devcontainer_labels = []
    for key, value in labels.items():
        devcontainer_labels.append(f"--label={key}={value}")
    return devcontainer_labels


def _get_devcontainer_file_path(directory_path: str) -> str:
    """Returns the path of the devcontainer in relation to provided directory path.

    :param directory_path: absolute path of local directory to write devcontainer.json.
    :type directory_path: str
    :return: Absolute path to the devcontainer
    :rtype: str
    """
    devcontainer_path = Path(directory_path, ".devcontainer")
    devcontainer_path.mkdir(parents=True, exist_ok=True)
    file_path = str(Path(devcontainer_path, "devcontainer.json").resolve())
    return file_path



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


from marshmallow import fields, post_load

from azure.ai.ml._schema.core.fields import ArmStr
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, AzureMLResourceType

from .artifact import ArtifactSchema


class IndexAssetSchema(ArtifactSchema):
    name = fields.Str(required=True, allow_none=False)
    id = ArmStr(azureml_type=AzureMLResourceType.INDEX, dump_only=True)
    stage = fields.Str(default="Development")
    path = fields.Str(
        required=True,
        metadata={
            "description": "A local path or a Blob URI pointing to a file or directory where index files are located."
        },
    )
    properties = fields.Dict(keys=fields.Str(), values=fields.Str())

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities._assets import Index

        return Index(base_path=self.context[BASE_PATH_CONTEXT_KEY], **data)



import pytest
from marshmallow.exceptions import ValidationError
from marshmallow.schema import Schema

from azure.ai.ml._schema import NestedField, UnionField
from azure.ai.ml._schema._deployment.online.scale_settings_schema import (
    DefaultScaleSettingsSchema,
    TargetUtilizationScaleSettingsSchema,
)
from azure.ai.ml._scope_dependent_operations import OperationScope


class DummySchema(Schema):
    scale_settings = UnionField(
        [NestedField(DefaultScaleSettingsSchema), NestedField(TargetUtilizationScaleSettingsSchema)]
    )


@pytest.mark.unittest
@pytest.mark.production_experiences_test
class TestScaleSettingsSchema:
    def test_default_scale_settings(self, mock_workspace_scope: OperationScope) -> None:
        schema = DummySchema()
        input_data = {"scale_settings": {"type": "default"}}
        schema.load(input_data)

    def test_target_utilization_scale_settings(self, mock_workspace_scope: OperationScope) -> None:
        schema = DummySchema()
        input_data = {"scale_settings": {"type": "target_utilization"}}
        schema.load(input_data)

    def test_scale_type_missing(self, mock_workspace_scope: OperationScope) -> None:
        schema = DummySchema()
        input_data = {"scale_settings": {}}
        with pytest.raises(ValidationError):
            schema.load(input_data)

    def test_invalid_scale_type(self, mock_workspace_scope: OperationScope) -> None:
        schema = DummySchema()
        input_data = {"scale_settings": {"type": "xxrei"}}
        with pytest.raises(ValidationError):
            schema.load(input_data)

    def test_invalid_fields(self, mock_workspace_scope: OperationScope) -> None:
        schema = DummySchema()
        input_data = {"scale_settings": {"type": "AuTo", "min_instances": 2, "max_instances": 1}}
        with pytest.raises(ValidationError):
            schema.load(input_data)

    def test_manual_missing_instance_count(self, mock_workspace_scope: OperationScope) -> None:
        schema = DummySchema()
        input_data = {"scale_settings": {"type": "Manual", "min_instances": 1, "max_instances": 2}}
        with pytest.raises(ValidationError):
            schema.load(input_data)

    def test_manual_normal_pass(self, mock_workspace_scope: OperationScope) -> None:
        schema = DummySchema()
        input_data = {
            "scale_settings": {
                "type": "target_utilization",
                "min_instances": 1,
                "max_instances": 2,
                "polling_interval": 1,
                "target_utilization_percentage": 80,
            }
        }
        data = schema.load(input_data)
        assert data["scale_settings"].type == "target_utilization"



# coding: utf-8
import lightgbm as lgb
import pandas as pd

if lgb.compat.MATPLOTLIB_INSTALLED:
    import matplotlib.pyplot as plt
else:
    raise ImportError("You need to install matplotlib for plot_example.py.")

print("Loading data...")
# load or create your dataset
df_train = pd.read_csv("../regression/regression.train", header=None, sep="\t")
df_test = pd.read_csv("../regression/regression.test", header=None, sep="\t")

y_train = df_train[0]
y_test = df_test[0]
X_train = df_train.drop(0, axis=1)
X_test = df_test.drop(0, axis=1)

# create dataset for lightgbm
lgb_train = lgb.Dataset(X_train, y_train)
lgb_test = lgb.Dataset(X_test, y_test, reference=lgb_train)

# specify your configurations as a dict
params = {"num_leaves": 5, "metric": ("l1", "l2"), "verbose": 0}

evals_result = {}  # to record eval results for plotting

print("Starting training...")
# train
gbm = lgb.train(
    params,
    lgb_train,
    num_boost_round=100,
    valid_sets=[lgb_train, lgb_test],
    feature_name=["f" + str(i + 1) for i in range(X_train.shape[-1])],
    categorical_feature=[21],
    evals_result=evals_result,
    verbose_eval=10,
)

print("Plotting metrics recorded during training...")
ax = lgb.plot_metric(evals_result, metric="l1")
plt.show()

print("Plotting feature importances...")
ax = lgb.plot_importance(gbm, max_num_features=10)
plt.show()

print("Plotting split value histogram...")
ax = lgb.plot_split_value_histogram(gbm, feature="f26", bins="auto")
plt.show()

print("Plotting 54th tree...")  # one tree use categorical feature to split
ax = lgb.plot_tree(gbm, tree_index=53, figsize=(15, 15), show_info=["split_gain"])
plt.show()

print("Plotting 54th tree with graphviz...")
graph = lgb.create_tree_digraph(gbm, tree_index=53, name="Tree54")
graph.render(view=True)



from pathlib import Path
from typing import Dict, Iterable, Optional
from unittest.mock import Mock, patch

import pytest

from azure.ai.ml import load_model
from azure.ai.ml._restclient.v2022_05_01.models._models_py3 import (
    ModelContainerData,
    ModelContainerDetails,
    ModelVersionData,
    ModelVersionDetails,
)
from azure.ai.ml._scope_dependent_operations import OperationConfig, OperationScope
from azure.ai.ml.entities._assets import Model
from azure.ai.ml.entities._assets._artifacts.artifact import ArtifactStorageInfo
from azure.ai.ml.exceptions import ErrorTarget, ValidationException
from azure.ai.ml.operations import DatastoreOperations, ModelOperations
from azure.core.exceptions import ResourceNotFoundError


@pytest.fixture
def mock_datastore_operation(
    mock_workspace_scope: OperationScope,
    mock_operation_config: OperationConfig,
    mock_aml_services_2024_01_01_preview: Mock,
    mock_aml_services_2024_07_01_preview: Mock,
) -> DatastoreOperations:
    yield DatastoreOperations(
        operation_scope=mock_workspace_scope,
        operation_config=mock_operation_config,
        serviceclient_2024_01_01_preview=mock_aml_services_2024_01_01_preview,
        serviceclient_2024_07_01_preview=mock_aml_services_2024_07_01_preview,
    )


@pytest.fixture
def mock_model_operation(
    mock_workspace_scope: OperationScope,
    mock_operation_config: OperationConfig,
    mock_aml_services_2022_05_01: Mock,
    mock_datastore_operation: Mock,
) -> ModelOperations:
    yield ModelOperations(
        operation_scope=mock_workspace_scope,
        operation_config=mock_operation_config,
        service_client=mock_aml_services_2022_05_01,
        datastore_operations=mock_datastore_operation,
    )


@pytest.fixture
def mock_model_operation_reg(
    mock_registry_scope: OperationScope,
    mock_operation_config: OperationConfig,
    mock_aml_services_2021_10_01_dataplanepreview: Mock,
    mock_datastore_operation: Mock,
) -> ModelOperations:
    yield ModelOperations(
        operation_scope=mock_registry_scope,
        operation_config=mock_operation_config,
        service_client=mock_aml_services_2021_10_01_dataplanepreview,
        datastore_operations=mock_datastore_operation,
    )


@pytest.mark.unittest
@pytest.mark.production_experiences_test
class TestModelOperations:
    def test_create_with_spec_file(
        self,
        mock_workspace_scope: OperationScope,
        mock_model_operation: ModelOperations,
        tmp_path: Path,
    ) -> None:
        model_name = f"model_random_string"
        p = tmp_path / "model_full.yml"
        model_path = tmp_path / "model.pkl"
        model_path.write_text("hello world")
        p.write_text(
            f"""
name: {model_name}
path: ./model.pkl
version: 3"""
        )

        with patch(
            "azure.ai.ml._artifacts._artifact_utilities._upload_to_datastore",
            return_value=ArtifactStorageInfo(
                name=model_name,
                version="3",
                relative_path="path",
                datastore_arm_id="/subscriptions/mock/resourceGroups/mock/providers/Microsoft.MachineLearningServices/workspaces/mock/datastores/datastore_id",
                container_name="containerName",
            ),
        ) as mock_upload, patch(
            "azure.ai.ml.operations._model_operations.Model._from_rest_object",
            return_value=Model(),
        ):
            model = load_model(source=p)
            path = Path(model._base_path, model.path).resolve()
            mock_model_operation.create_or_update(model)
            mock_upload.assert_called_once_with(
                mock_workspace_scope,
                mock_model_operation._datastore_operation,
                path,
                asset_name=model.name,
                asset_version=model.version,
                datastore_name=None,
                asset_hash=None,
                sas_uri=None,
                artifact_type=ErrorTarget.MODEL,
                show_progress=True,
                ignore_file=None,
                blob_uri=None,
            )
        mock_model_operation._model_versions_operation.create_or_update.assert_called_once()
        assert "version='3'" in str(mock_model_operation._model_versions_operation.create_or_update.call_args)

    def test_create_autoincrement(
        self,
        mock_model_operation: ModelOperations,
        mock_workspace_scope: OperationScope,
        tmp_path: Path,
    ) -> None:
        model_name = f"model_random_string"
        p = tmp_path / "model_full.yml"
        model_path = tmp_path / "model.pkl"
        model_path.write_text("hello world")
        p.write_text(
            f"""
name: {model_name}
path: ./model.pkl"""
        )
        model = load_model(source=p)
        assert model._auto_increment_version
        model.version = None

        with patch("azure.ai.ml.operations._model_operations.Model._from_rest_object", return_value=None), patch(
            "azure.ai.ml.operations._model_operations._get_next_version_from_container", return_value="version"
        ) as mock_nextver, patch(
            "azure.ai.ml.operations._model_operations._check_and_upload_path",
            return_value=(model, "indicatorfile.txt"),
        ), patch(
            "azure.ai.ml.operations._model_operations.Model._from_rest_object", return_value=model
        ), patch(
            "azure.ai.ml.operations._model_operations._get_default_datastore_info", return_value=None
        ):
            mock_model_operation.create_or_update(model)
            mock_nextver.assert_called_once()

            mock_model_operation._model_versions_operation.create_or_update.assert_called_once_with(
                body=model._to_rest_object(),
                name=model.name,
                version=mock_nextver.return_value,
                resource_group_name=mock_workspace_scope.resource_group_name,
                workspace_name=mock_workspace_scope.workspace_name,
            )

    def test_get_name_and_version(self, mock_model_operation: ModelOperations) -> None:
        mock_model_operation._model_container_operation.get.return_value = None
        with patch(
            "azure.ai.ml.operations._model_operations.Model._from_rest_object",
            return_value=None,
        ):
            mock_model_operation.get(name="random_string", version="1")
        mock_model_operation._model_versions_operation.get.assert_called_once()
        assert mock_model_operation._model_container_operation.get.call_count == 0

    def test_get_no_version(self, mock_model_operation: ModelOperations) -> None:
        name = "random_string"
        with pytest.raises(Exception):
            mock_model_operation.get(name=name)

    @patch.object(Model, "_from_rest_object", new=Mock())
    @patch.object(Model, "_from_container_rest_object", new=Mock())
    def test_list(self, mock_model_operation: ModelOperations) -> None:
        mock_model_operation._model_versions_operation.list.return_value = [Mock(Model) for _ in range(10)]
        mock_model_operation._model_container_operation.list.return_value = [Mock(Model) for _ in range(10)]
        result = mock_model_operation.list()
        assert isinstance(result, Iterable)
        mock_model_operation._model_container_operation.list.assert_called_once()
        mock_model_operation.list(name="random_string")
        mock_model_operation._model_versions_operation.list.assert_called_once()

    def test_archive_version(self, mock_model_operation: ModelOperations) -> None:
        name = "random_string"
        model_version = Mock(ModelVersionData(properties=Mock(ModelVersionDetails())))
        version = "1"
        mock_model_operation._model_versions_operation.get.return_value = model_version
        mock_model_operation.archive(name=name, version=version)
        mock_model_operation._model_versions_operation.create_or_update.assert_called_once_with(
            name=name,
            version=version,
            workspace_name=mock_model_operation._workspace_name,
            body=model_version,
            resource_group_name=mock_model_operation._resource_group_name,
        )

    def test_archive_container(self, mock_model_operation: ModelOperations) -> None:
        name = "random_string"
        model_container = Mock(ModelContainerData(properties=Mock(ModelContainerDetails())))
        mock_model_operation._model_container_operation.get.return_value = model_container
        mock_model_operation.archive(name=name)
        mock_model_operation._model_container_operation.create_or_update.assert_called_once_with(
            name=name,
            workspace_name=mock_model_operation._workspace_name,
            body=model_container,
            resource_group_name=mock_model_operation._resource_group_name,
        )

    def test_restore_version(self, mock_model_operation: ModelOperations) -> None:
        name = "random_string"
        model = Mock(ModelVersionData(properties=Mock(ModelVersionDetails())))
        version = "1"
        mock_model_operation._model_versions_operation.get.return_value = model
        mock_model_operation.restore(name=name, version=version)
        mock_model_operation._model_versions_operation.create_or_update.assert_called_with(
            name=name,
            version=version,
            workspace_name=mock_model_operation._workspace_name,
            body=model,
            resource_group_name=mock_model_operation._resource_group_name,
        )

    def test_restore_container(self, mock_model_operation: ModelOperations) -> None:
        name = "random_string"
        model_container = Mock(ModelContainerData(properties=Mock(ModelContainerDetails())))
        mock_model_operation._model_container_operation.get.return_value = model_container
        mock_model_operation.restore(name=name)
        mock_model_operation._model_container_operation.create_or_update.assert_called_once_with(
            name=name,
            workspace_name=mock_model_operation._workspace_name,
            body=model_container,
            resource_group_name=mock_model_operation._resource_group_name,
        )

    def test_download_from_gen2_with_none_cred(self, mock_model_operation: ModelOperations) -> None:
        name = "random_string"
        version = "1"
        model = Model(
            name=name,
            version=version,
            path="azureml://subscriptions/subscription_id/resourcegroups/rg-name/workspaces/gen2test/datastores/adls_gen2/paths/gen2test/",
        )
        from azure.ai.ml.entities import AzureDataLakeGen2Datastore

        datastore = AzureDataLakeGen2Datastore(name="gen2_datastore", account_name="gen2_account", filesystem="gen2")
        storage_client = Mock()
        with patch(
            "azure.ai.ml.operations._model_operations.get_storage_client", return_value=storage_client
        ) as get_client_mock, patch(
            "azure.ai.ml.operations._model_operations.Model._from_rest_object",
            return_value=model,
        ), patch(
            "azure.ai.ml.operations._model_operations.DatastoreOperations.get", return_value=datastore
        ):
            mock_model_operation.download(name=name, version=version)
            get_client_mock.assert_called_once()

    def test_download_from_gen2_with_sp_cred(self, mock_model_operation: ModelOperations) -> None:
        name = "random_string"
        version = "1"
        model = Model(
            name=name,
            version=version,
            path="azureml://subscriptions/subscription_id/resourcegroups/rg-name/workspaces/gen2test/datastores/adls_gen2/paths/gen2test/",
        )
        from azure.ai.ml.entities import AzureDataLakeGen2Datastore
        from azure.ai.ml.entities._credentials import ServicePrincipalConfiguration

        datastore = AzureDataLakeGen2Datastore(
            name="adls_gen2_example",
            description="Datastore pointing to an Azure Data Lake Storage Gen2.",
            account_name="mytestdatalakegen2",
            filesystem="my-gen2-container",
            credentials=ServicePrincipalConfiguration(
                tenant_id="XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX",
                client_id="XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX",
                client_secret="XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
            ),
        )
        storage_client = Mock()
        with patch(
            "azure.ai.ml.operations._model_operations.get_storage_client", return_value=storage_client
        ) as get_client_mock, patch(
            "azure.ai.ml.operations._model_operations.Model._from_rest_object",
            return_value=model,
        ), patch(
            "azure.ai.ml.operations._model_operations.DatastoreOperations.get", return_value=datastore
        ):
            mock_model_operation.download(name=name, version=version)
            get_client_mock.assert_called_once()

    def test_create_with_datastore(
        self,
        mock_workspace_scope: OperationScope,
        mock_model_operation: ModelOperations,
    ) -> None:
        p = "./tests/test_configs/model/model_with_datastore.yml"
        model_name = f"model_random_string"

        with patch(
            "azure.ai.ml._artifacts._artifact_utilities._upload_to_datastore",
            return_value=ArtifactStorageInfo(
                name=model_name,
                version="3",
                relative_path="path",
                datastore_arm_id="/subscriptions/mock/resourceGroups/mock/providers/Microsoft.MachineLearningServices/workspaces/mock/datastores/datastore_id",
                container_name="containerName",
            ),
        ) as mock_upload, patch(
            "azure.ai.ml.operations._model_operations.Model._from_rest_object",
            return_value=Model(),
        ):
            model = load_model(p)
            path = Path(model._base_path, model.path).resolve()
            mock_model_operation.create_or_update(model)
            mock_upload.assert_called_once_with(
                mock_workspace_scope,
                mock_model_operation._datastore_operation,
                path,
                asset_name=model.name,
                asset_version=model.version,
                datastore_name="workspaceartifactstore",
                asset_hash=None,
                sas_uri=None,
                artifact_type=ErrorTarget.MODEL,
                show_progress=True,
                ignore_file=None,
                blob_uri=None,
            )

    # def test_promote_model_from_workspace(
    #     self,
    #     mock_model_operation_reg: ModelOperations,
    #     mock_model_operation: ModelOperations,
    #     tmp_path: Path,
    # ) -> None:
    #     model_name = f"model_random_string"
    #     p = tmp_path / "model_full.yml"
    #     model_path = tmp_path / "model.pkl"
    #     model_path.write_text("hello world")
    #     p.write_text(
    #         f"""
    # name: {model_name}
    # path: ./model.pkl
    # version: 3"""
    #     )

    #     with patch(
    #         "azure.ai.ml._artifacts._artifact_utilities._upload_to_datastore",
    #         return_value=ArtifactStorageInfo(
    #             name=model_name,
    #             version="3",
    #             relative_path="path",
    #             datastore_arm_id="/subscriptions/mock/resourceGroups/mock/providers/Microsoft.MachineLearningServices/workspaces/mock/datastores/datastore_id",
    #             container_name="containerName",
    #         ),
    #     ) as mock_upload, patch(
    #         "azure.ai.ml.operations._model_operations.Model._from_rest_object",
    #         return_value=Model(),
    #     ):
    #         model = load_model(source=p)
    #         model_to_promote = mock_model_operation._prepare_to_copy(model, "new_name", "new_version")
    #         assert model_to_promote.name == "new_name"
    #         assert model_to_promote.version == "new_version"
    #         mock_model_operation_reg._model_versions_operation.get.side_effect = Mock(
    #             side_effect=ResourceNotFoundError("Test")
    #         )
    #         mock_model_operation_reg.create_or_update(model_to_promote)
    #         mock_model_operation_reg._service_client.resource_management_asset_reference.begin_import_method.assert_called_once()

    def test_model_entity_class_exist(self):
        try:
            from azure.ai.ml.entities import WorkspaceModelReference
        except ImportError:
            assert False, "WorkspaceModelReference class not found"

    @pytest.mark.parametrize(
        "old_properties,new_properties",
        [
            (None, {}),
            ({"test": "test"}, {}),
            ({}, {"test": "test"}),
            ({}, {}),
            ({"is-promptflow": "true", "is-evaluator": "true"}, {"is-promptflow": "true", "is-evaluator": "true"}),
            (None, {"is-promptflow": "true", "is-evaluator": "true"}),
        ],
    )
    def test_create_success(
        self,
        mock_datastore_operation: DatastoreOperations,
        mock_operation_config: OperationConfig,
        mock_workspace_scope: OperationScope,
        tmp_path: Path,
        old_properties: Dict[str, str],
        new_properties: Dict[str, str],
    ):
        mock_model_operation = ModelOperations(
            operation_scope=mock_workspace_scope,
            operation_config=mock_operation_config,
            service_client=Mock(),
            datastore_operations=mock_datastore_operation,
            **{ModelOperations._IS_EVALUATOR: True},
        )
        """Test that new version is created if models are of the same type."""
        model_name = f"model_random_string"
        p = tmp_path / "model_full.yml"
        model_path = tmp_path / "model.pkl"
        model_path.write_text("hello world")
        p.write_text(
            f"""
name: {model_name}
path: ./model.pkl
version: 3"""
        )

        with patch(
            "azure.ai.ml._artifacts._artifact_utilities._upload_to_datastore",
            return_value=ArtifactStorageInfo(
                name=model_name,
                version="3",
                relative_path="path",
                datastore_arm_id="/subscriptions/mock/resourceGroups/mock/providers/Microsoft.MachineLearningServices/workspaces/mock/datastores/datastore_id",
                container_name="containerName",
            ),
        ) as mock_upload, patch(
            "azure.ai.ml.operations._model_operations.Model._from_rest_object", return_value=Model()
        ), patch(
            "azure.ai.ml.operations._model_operations.ModelOperations._get_model_properties",
            return_value=old_properties,
        ):
            model = load_model(source=p)
            model.properties = new_properties
            path = Path(model._base_path, model.path).resolve()
            mock_model_operation.create_or_update(model)
            mock_upload.assert_called_once_with(
                mock_workspace_scope,
                mock_model_operation._datastore_operation,
                path,
                asset_name=model.name,
                asset_version=model.version,
                datastore_name=None,
                asset_hash=None,
                sas_uri=None,
                artifact_type=ErrorTarget.MODEL,
                show_progress=True,
                ignore_file=None,
                blob_uri=None,
            )
        mock_model_operation._model_versions_operation.create_or_update.assert_called_once()

    @pytest.mark.parametrize(
        "old_properties,new_properties,message",
        [
            # ({"is-promptflow": "true", "is-evaluator": "true"}, {}, "because previous version of model was marked"),
            ({}, {"is-promptflow": "true", "is-evaluator": "true"}, "because this version of model was marked"),
        ],
    )
    def test_create_raises_if_wrong_type(
        self,
        mock_datastore_operation: DatastoreOperations,
        mock_operation_config: OperationConfig,
        mock_workspace_scope: OperationScope,
        tmp_path: Path,
        old_properties: Dict[str, str],
        new_properties: Dict[str, str],
        message: str,
    ) -> None:
        """Test exception if pre existing model is not of a correct type."""
        mock_model_operation = ModelOperations(
            operation_scope=mock_workspace_scope,
            operation_config=mock_operation_config,
            service_client=Mock(),
            datastore_operations=mock_datastore_operation,
            **{ModelOperations._IS_EVALUATOR: True},
        )
        model_name = f"model_random_string"
        p = tmp_path / "model_full.yml"
        model_path = tmp_path / "model.pkl"
        model_path.write_text("hello world")
        p.write_text(
            f"""
name: {model_name}
path: ./model.pkl"""
        )
        new_model = load_model(source=p)
        new_model.properties = new_properties
        with patch(
            "azure.ai.ml.operations._model_operations.ModelOperations._get_model_properties",
            return_value=old_properties,
        ):
            with pytest.raises(ValidationException) as cm:
                mock_model_operation.create_or_update(new_model)
            assert message in cm.value.args[0]

    @pytest.mark.parametrize(
        "label,version,get_raises,latest_raises,expected",
        [
            ("lbl", None, False, False, {"model": "from_get"}),
            ("lbl", "1", False, False, {"model": "from_get"}),
            (None, "1", False, False, {"model": "from_get"}),
            (None, None, False, False, {"model": "from_latest"}),
            ("lbl", None, True, False, None),
            (None, "1", True, False, None),
            (None, None, True, False, {"model": "from_latest"}),
            (None, None, True, True, None),
        ],
    )
    def test_return_properties(
        self,
        mock_model_operation: ModelOperations,
        tmp_path: Path,
        label: Optional[str],
        version: Optional[str],
        get_raises: bool,
        latest_raises: bool,
        expected: Optional[Dict[str, str]],
    ) -> None:
        model_name = f"model_random_string"
        p = tmp_path / "model_full.yml"
        model_path = tmp_path / "model.pkl"
        model_path.write_text("hello world")
        p.write_text(
            f"""
name: {model_name}
path: ./model.pkl"""
        )
        get_model = load_model(source=p)
        get_model.properties = {"model": "from_get"}
        latest_model = load_model(source=p)
        latest_model.properties = {"model": "from_latest"}
        get_kw = {"side_effect": ResourceNotFoundError("Mock") if get_raises else None, "return_value": get_model}
        get_latest = {
            "side_effect": ResourceNotFoundError("Mock") if latest_raises else None,
            "return_value": latest_model,
        }
        with patch("azure.ai.ml.operations._model_operations.ModelOperations.get", **get_kw):
            with patch("azure.ai.ml.operations._model_operations.ModelOperations._get_latest_version", **get_latest):
                assert mock_model_operation._get_model_properties(model_name, version, label) == expected

    def test_model_operation_raises_on_evaluators(self, mock_model_operation: ModelOperations, tmp_path: Path):
        """Test model_operation raiese if evaluator is being created."""
        model_name = f"model_random_string"
        p = tmp_path / "model_full.yml"
        model_path = tmp_path / "model.pkl"
        model_path.write_text("hello world")
        p.write_text(
            f"""
name: {model_name}
path: ./model.pkl"""
        )
        model = load_model(source=p)
        model.properties = {"is-promptflow": "true", "is-evaluator": "true"}
        with pytest.raises(ValidationException) as cm:
            mock_model_operation.create_or_update(model)
        assert "please use EvaluatorOperations" in cm.value.args[0]



import argparse
import os
from datetime import datetime

print("Hello Python World")

parser = argparse.ArgumentParser()
parser.add_argument("--input_data", type=str)
parser.add_argument("--input_string", type=str)
parser.add_argument("--output_data", type=str)

args = parser.parse_args()

print("sample_input_string: %s" % args.input_string)
print("sample_input_data path: %s" % args.input_data)
print("sample_output_data path: %s" % args.output_data)

print("files in input_data path: ")
arr = os.listdir(args.input_data)
print(arr)

for filename in arr:
    print("reading file: %s ..." % filename)
    with open(os.path.join(args.input_data, filename), "r") as handle:
        print(handle.read())

cur_time_str = datetime.now().strftime("%b-%d-%Y-%H-%M-%S")

print("Writing file: %s" % os.path.join(args.output_data, "file-" + cur_time_str + ".txt"))
with open(os.path.join(args.output_data, "file-" + cur_time_str + ".txt"), "wt") as text_file:
    print(f"Logging date time: {cur_time_str}", file=text_file)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Optional

import pytest
from azure.ai.ml.entities import CustomModelFineTuningJob
from devtools_testutils import AzureRecordedTestCase, is_live

from azure.ai.ml import MLClient
from test_common_functions import validate_job


@pytest.mark.finetuning_job_test
@pytest.mark.usefixtures("recorded_test")
@pytest.mark.skipif(condition=not is_live(), reason="This test requires a live endpoint")
class TestCustomModelFineTuningJob_MaaP(AzureRecordedTestCase):

    @pytest.mark.e2etest
    def test_using_instance_types(
        self,
        finetuning_job_instance_types: CustomModelFineTuningJob,
        client: MLClient,
    ) -> None:

        # Act
        created_job = client.jobs.create_or_update(finetuning_job_instance_types)
        returned_job = client.jobs.get(created_job.name)

        # Assert
        validate_job(
            input_job=finetuning_job_instance_types,
            created_job=created_job,
            returned_job=returned_job,
        )

    @pytest.mark.e2etest
    def test_using_aml_compute(
        self,
        finetuning_job_compute: CustomModelFineTuningJob,
        client: MLClient,
    ) -> None:
        # Act
        created_job = client.jobs.create_or_update(finetuning_job_compute)
        returned_job = client.jobs.get(created_job.name)

        # Assert
        validate_job(
            input_job=finetuning_job_compute,
            created_job=created_job,
            returned_job=returned_job,
        )

    @pytest.mark.e2etest
    def test_using_queue_settings(
        self,
        finetuning_job_queue_settings: CustomModelFineTuningJob,
        client: MLClient,
    ) -> None:

        # Act
        created_job = client.jobs.create_or_update(finetuning_job_queue_settings)
        returned_job = client.jobs.get(created_job.name)

        # Assert
        validate_job(
            input_job=finetuning_job_queue_settings,
            created_job=created_job,
            returned_job=returned_job,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from os import PathLike
from pathlib import Path
from typing import Any, Dict, Optional, Union

from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    FlavorData,
    ModelContainer,
    ModelVersion,
    ModelVersionProperties,
)
from azure.ai.ml._schema import ModelSchema
from azure.ai.ml._utils._arm_id_utils import AMLNamedArmId, AMLVersionedArmId
from azure.ai.ml._utils._asset_utils import get_ignore_file, get_object_hash
from azure.ai.ml.constants._common import (
    BASE_PATH_CONTEXT_KEY,
    LONG_URI_FORMAT,
    PARAMS_OVERRIDE_KEY,
    ArmConstants,
    AssetTypes,
)
from azure.ai.ml.entities._assets import Artifact
from azure.ai.ml.entities._assets.intellectual_property import IntellectualProperty
from azure.ai.ml.entities._system_data import SystemData
from azure.ai.ml.entities._util import get_md5_string, load_from_dict

from .artifact import ArtifactStorageInfo


class Model(Artifact):  # pylint: disable=too-many-instance-attributes
    """Model for training and scoring.

    :param name: The name of the model. Defaults to a random GUID.
    :type name: Optional[str]
    :param version: The version of the model. Defaults to "1" if either no name or an unregistered name is provided.
        Otherwise, defaults to autoincrement from the last registered version of the model with that name.
    :type version: Optional[str]
    :param type: The storage format for this entity, used for NCD (Novel Class Discovery). Accepted values are
        "custom_model", "mlflow_model", or "triton_model". Defaults to "custom_model".
    :type type: Optional[str]
    :param utc_time_created: The date and time when the model was created, in
        UTC ISO 8601 format. (e.g. '2020-10-19 17:44:02.096572').
    :type utc_time_created: Optional[str]
    :param flavors: The flavors in which the model can be interpreted. Defaults to None.
    :type flavors: Optional[dict[str, Any]]
    :param path: A remote uri or a local path pointing to a model. Defaults to None.
    :type path: Optional[str]
    :param description: The description of the resource. Defaults to None
    :type description: Optional[str]
    :param tags: Tag dictionary. Tags can be added, removed, and updated. Defaults to None.
    :type tags: Optional[dict[str, str]]
    :param properties: The asset property dictionary. Defaults to None.
    :type properties: Optional[dict[str, str]]
    :param stage: The stage of the resource. Defaults to None.
    :type stage: Optional[str]
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: Optional[dict]

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_misc.py
            :start-after: [START model_entity_create]
            :end-before: [END model_entity_create]
            :language: python
            :dedent: 8
            :caption: Creating a Model object.
    """

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        version: Optional[str] = None,
        type: Optional[str] = None,  # pylint: disable=redefined-builtin
        path: Optional[Union[str, PathLike]] = None,
        utc_time_created: Optional[str] = None,
        flavors: Optional[Dict[str, Dict[str, Any]]] = None,
        description: Optional[str] = None,
        tags: Optional[Dict] = None,
        properties: Optional[Dict] = None,
        stage: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        self.job_name = kwargs.pop("job_name", None)
        self._intellectual_property = kwargs.pop("intellectual_property", None)
        self._system_metadata = kwargs.pop("system_metadata", None)
        super().__init__(
            name=name,
            version=version,
            path=path,
            description=description,
            tags=tags,
            properties=properties,
            **kwargs,
        )
        self.utc_time_created = utc_time_created
        self.flavors = dict(flavors) if flavors else None
        self._arm_type = ArmConstants.MODEL_VERSION_TYPE
        self.type = type or AssetTypes.CUSTOM_MODEL
        self.stage = stage
        if self._is_anonymous and self.path:
            _ignore_file = get_ignore_file(self.path)
            _upload_hash = get_object_hash(self.path, _ignore_file)
            self.name = get_md5_string(_upload_hash)

    @classmethod
    def _load(
        cls,
        data: Optional[Dict] = None,
        yaml_path: Optional[Union[PathLike, str]] = None,
        params_override: Optional[list] = None,
        **kwargs: Any,
    ) -> "Model":
        params_override = params_override or []
        data = data or {}
        context = {
            BASE_PATH_CONTEXT_KEY: Path(yaml_path).parent if yaml_path else Path("./"),
            PARAMS_OVERRIDE_KEY: params_override,
        }
        res: Model = load_from_dict(ModelSchema, data, context, **kwargs)
        return res

    def _to_dict(self) -> Dict:
        return dict(ModelSchema(context={BASE_PATH_CONTEXT_KEY: "./"}).dump(self))

    @classmethod
    def _from_rest_object(cls, model_rest_object: ModelVersion) -> "Model":
        rest_model_version: ModelVersionProperties = model_rest_object.properties
        arm_id = AMLVersionedArmId(arm_id=model_rest_object.id)
        model_stage = rest_model_version.stage if hasattr(rest_model_version, "stage") else None
        model_system_metadata = (
            rest_model_version.system_metadata if hasattr(rest_model_version, "system_metadata") else None
        )
        if hasattr(rest_model_version, "flavors"):
            flavors = {key: flavor.data for key, flavor in rest_model_version.flavors.items()}
        model = Model(
            id=model_rest_object.id,
            name=arm_id.asset_name,
            version=arm_id.asset_version,
            path=rest_model_version.model_uri,
            description=rest_model_version.description,
            tags=rest_model_version.tags,
            flavors=flavors,  # pylint: disable=possibly-used-before-assignment
            properties=rest_model_version.properties,
            stage=model_stage,
            # pylint: disable=protected-access
            creation_context=SystemData._from_rest_object(model_rest_object.system_data),
            type=rest_model_version.model_type,
            job_name=rest_model_version.job_name,
            intellectual_property=(
                IntellectualProperty._from_rest_object(rest_model_version.intellectual_property)
                if rest_model_version.intellectual_property
                else None
            ),
            system_metadata=model_system_metadata,
        )
        return model

    @classmethod
    def _from_container_rest_object(cls, model_container_rest_object: ModelContainer) -> "Model":
        model = Model(
            name=model_container_rest_object.name,
            version="1",
            id=model_container_rest_object.id,
            # pylint: disable=protected-access
            creation_context=SystemData._from_rest_object(model_container_rest_object.system_data),
        )
        model.latest_version = model_container_rest_object.properties.latest_version

        # Setting version to None since if version is not provided it is defaulted to "1".
        # This should go away once container concept is finalized.
        model.version = None
        return model

    def _to_rest_object(self) -> ModelVersion:
        model_version = ModelVersionProperties(
            description=self.description,
            tags=self.tags,
            properties=self.properties,
            flavors=(
                {key: FlavorData(data=dict(value)) for key, value in self.flavors.items()} if self.flavors else None
            ),  # flatten OrderedDict to dict
            model_type=self.type,
            model_uri=self.path,
            stage=self.stage,
            is_anonymous=self._is_anonymous,
        )
        model_version.system_metadata = self._system_metadata if hasattr(self, "_system_metadata") else None

        model_version_resource = ModelVersion(properties=model_version)

        return model_version_resource

    def _update_path(self, asset_artifact: ArtifactStorageInfo) -> None:
        # datastore_arm_id is null for registry scenario, so capture the full_storage_path
        if not asset_artifact.datastore_arm_id and asset_artifact.full_storage_path:
            self.path = asset_artifact.full_storage_path
        else:
            aml_datastore_id = AMLNamedArmId(asset_artifact.datastore_arm_id)
            self.path = LONG_URI_FORMAT.format(
                aml_datastore_id.subscription_id,
                aml_datastore_id.resource_group_name,
                aml_datastore_id.workspace_name,
                aml_datastore_id.asset_name,
                asset_artifact.relative_path,
            )

    def _to_arm_resource_param(self, **kwargs: Any) -> Dict:  # pylint: disable=unused-argument
        properties = self._to_rest_object().properties

        return {
            self._arm_type: {
                ArmConstants.NAME: self.name,
                ArmConstants.VERSION: self.version,
                ArmConstants.PROPERTIES_PARAMETER_NAME: self._serialize.body(properties, "ModelVersionProperties"),
            }
        }



from pathlib import Path

from azure.ai.ml import Input, Output, dsl, load_component
from azure.ai.ml.entities import PipelineJob

parent_dir = str(Path(__file__).parent)


def generate_dsl_pipeline() -> PipelineJob:
    # 1. Load component funcs
    prep_func = load_component(source=parent_dir + "/prep.yml")
    transform_func = load_component(source=parent_dir + "/transform.yml")
    train_func = load_component(source=parent_dir + "/train.yml")
    predict_func = load_component(source=parent_dir + "/predict.yml")
    score_func = load_component(source=parent_dir + "/score.yml")

    # 2. Construct pipeline
    @dsl.pipeline(compute="cpu-cluster", default_datastore="workspaceblobstore")
    def nyc_taxi_data_regression(pipeline_job_input):
        prep_job = prep_func(raw_data=pipeline_job_input)
        transform_job = transform_func(clean_data=prep_job.outputs.prep_data)
        train_job = train_func(training_data=transform_job.outputs.transformed_data)
        predict_job = predict_func(model_input=train_job.outputs.model_output, test_data=train_job.outputs.test_data)
        score_job = score_func(predictions=predict_job.outputs.predictions, model=train_job.outputs.model_output)
        return {
            "pipeline_job_prepped_data": prep_job.outputs.prep_data,
            "pipeline_job_transformed_data": transform_job.outputs.transformed_data,
            "pipeline_job_trained_model": train_job.outputs.model_output,
            "pipeline_job_test_data": train_job.outputs.test_data,
            "pipeline_job_predictions": predict_job.outputs.predictions,
            "pipeline_job_score_report": score_job.outputs.score_report,
        }

    pipeline = nyc_taxi_data_regression(Input(path=parent_dir + "/data/"))
    pipeline.outputs.pipeline_job_prepped_data = Output(path="/prepped_data", mode="rw_mount")
    pipeline.outputs.pipeline_job_transformed_data = Output(path="/transformed_data", mode="rw_mount")
    pipeline.outputs.pipeline_job_trained_model = Output(path="/trained-model", mode="rw_mount")
    pipeline.outputs.pipeline_job_test_data = Output(path="/test_data", mode="rw_mount")
    pipeline.outputs.pipeline_job_predictions = Output(path="/predictions", mode="rw_mount")
    pipeline.outputs.pipeline_job_score_report = Output(path="/report", mode="rw_mount")
    return pipeline



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from marshmallow import fields, post_load
from azure.ai.ml._schema.core.fields import StringTransformedEnum
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta
from azure.ai.ml._utils._experimental import experimental
from azure.ai.ml._utils.utils import camel_to_snake
from azure.ai.ml.constants._common import AutoDeleteCondition
from azure.ai.ml.entities._assets.auto_delete_setting import AutoDeleteSetting


@experimental
class BaseAutoDeleteSettingSchema(metaclass=PatchedSchemaMeta):
    @post_load
    def make(self, data, **kwargs) -> "AutoDeleteSetting":
        return AutoDeleteSetting(**data)


@experimental
class AutoDeleteConditionSchema(BaseAutoDeleteSettingSchema):
    condition = StringTransformedEnum(
        allowed_values=[condition.name for condition in AutoDeleteCondition],
        casing_transform=camel_to_snake,
    )


@experimental
class ValueSchema(BaseAutoDeleteSettingSchema):
    value = fields.Str()


@experimental
class AutoDeleteSettingSchema(AutoDeleteConditionSchema, ValueSchema):
    pass



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

""" This is logger utility which will work with allowed logged filter AML policy
    https://github.com/Azure/azure-policy/blob/master/built-in-policies/policyDefinitions/Machine%20Learning/AllowedLogFilter_EnforceSetting.json
    You have to define the same "logFilters" while initializing the logger using  "enable_compliant_logging" method
    e.g. 
        log filters: ["^SystemLog:.*$"]
        initialize : enable_compliant_logging(format_key="prefix",
                                format_key_value="SystemLog",
                                format=f"%(prefix)s{logging.BASIC_FORMAT}")
    By default log message will not compliant e.g. not modified
"""

import logging
import sys
from datetime import datetime
from enum import Enum
from threading import Lock
from typing import Optional

_LOCK = Lock()
_FORMAT_KEY = None
_FORMAT_VALUE = None


# pylint: disable=global-statement
def set_format(key_name: str, value: str) -> None:
    with _LOCK:
        global _FORMAT_KEY
        _FORMAT_KEY = key_name
        global _FORMAT_VALUE
        _FORMAT_VALUE = value


def get_format_key() -> Optional[str]:
    return _FORMAT_KEY


def get_format_value() -> Optional[str]:
    return _FORMAT_VALUE


def get_default_logging_format() -> str:
    return f"%({get_format_key()})s{logging.BASIC_FORMAT}"


class DataCategory(Enum):
    """
    Enumeration of data categories in compliant machine learning.

    Values:
    - PRIVATE: data which is private. Researchers may not view this.
    - PUBLIC: data which may safely be viewed by researchers.
    """

    PRIVATE = 1
    PUBLIC = 2


class CompliantLogger(logging.getLoggerClass()):  # type: ignore
    """
    Subclass of the default logging class with an explicit `is_compliant` parameter
    on all logging methods. It will pass an `extra` param with `format` key
    (value depending on whether `is_compliant` is True or False) to the
    handlers.

    The default value for data `is_compliant` is `False` for all methods.

    Implementation is inspired by:
    https://github.com/python/cpython/blob/3.8/Lib/logging/__init__.py
    """

    def __init__(self, name: str, handlers=None):
        super().__init__(name)  # type: ignore

        self.format_key = get_format_key()
        self.format_value = get_format_value()

        if handlers:
            self.handlers = handlers

        self.start_time = datetime.now()
        self.metric_count = 1
        # number of iterable items that are logged
        self.max_iter_items = 10

    def _log(
        self,
        level,
        msg,
        args=None,
        exc_info=None,
        extra=None,
        stack_info=False,
        stacklevel=1,
        category=DataCategory.PRIVATE,
    ):
        if category == DataCategory.PUBLIC:
            format_value = self.format_value
        else:
            format_value = ""

        if extra:
            extra.update({self.format_key: format_value})
        else:
            extra = {self.format_key: format_value}

        if sys.version_info[1] <= 7:
            super(CompliantLogger, self)._log(
                level=level,
                msg=msg,
                args=args,
                exc_info=exc_info,
                extra=extra,
                stack_info=stack_info,
            )
        else:
            super(CompliantLogger, self)._log(
                level=level,
                msg=msg,
                args=args,
                exc_info=exc_info,
                extra=extra,
                stack_info=stack_info,
                stacklevel=stacklevel,  # type: ignore
            )


_logging_basic_config_set_warning = """
********************************************************************************
The root logger already has handlers set! As a result, the behavior of this
library is undefined. If running in Python >= 3.8, this library will attempt to
call logging.basicConfig(force=True), which will remove all existing root
handlers. See https://stackoverflow.com/q/20240464 and
https://github.com/Azure/confidential-ml-utils/issues/33 for more information.
********************************************************************************
"""


def enable_compliant_logging(
    format_key: str = "prefix",
    format_key_value: str = "SystemLog:",
    **kwargs,
) -> None:
    """
    The default format is `logging.BASIC_FORMAT` (`%(levelname)s:%(name)s:%(message)s`).
    All other kwargs are passed to `logging.basicConfig`. Sets the default
    logger class and root logger to be compliant. This means the format
    string `%(xxxx)` will work.

    :param format_key: key for format
    :type format_key: str
    :param format_key_value: value for format
    :type format_key_value: str

    Set the format using the `format` kwarg.

    If running in Python >= 3.8, will attempt to add `force=True` to the kwargs
    for logging.basicConfig.

    The standard implementation of the logging API is a good reference:
    https://github.com/python/cpython/blob/3.9/Lib/logging/__init__.py
    """
    set_format(format_key, format_key_value)

    if "format" not in kwargs:
        kwargs["format"] = get_default_logging_format()

    # Ensure that all loggers created via `logging.getLogger` are instances of
    # the `CompliantLogger` class.
    logging.setLoggerClass(CompliantLogger)

    if len(logging.root.handlers) > 0:
        p = get_format_value()
        for line in _logging_basic_config_set_warning.splitlines():
            print(f"{p}{line}", file=sys.stderr)

    if "force" not in kwargs and sys.version_info >= (3, 8):
        kwargs["force"] = True

    root = CompliantLogger(logging.root.name, handlers=logging.root.handlers)

    logging.root = root
    logging.Logger.root = root  # type: ignore
    logging.Logger.manager = logging.Manager(root)  # type: ignore

    # https://github.com/kivy/kivy/issues/6733
    logging.basicConfig(**kwargs)



import json
import os
from pathlib import Path
from typing import Dict

import pytest

from azure.ai.ml._utils._data_utils import download_mltable_metadata_schema, validate_mltable_metadata
from azure.ai.ml._utils.utils import load_yaml
from azure.ai.ml.constants._common import MLTABLE_METADATA_SCHEMA_URL_FALLBACK
from azure.ai.ml.exceptions import ValidationException


@pytest.fixture
def mltable_schema():
    with open(Path(os.path.dirname(__file__), "../../../../../", "extension/schema/MLTable.schema.json"), "r") as f:
        return json.load(f)


@pytest.mark.unittest
@pytest.mark.skip(reason="extension folder does not exist in GitHub")
class TestDataUtils:
    def test_validate_mltable_metadata_schema(self, tmp_path: Path, mltable_schema: Dict):
        mltable_folder = tmp_path / "mltable_folder"
        mltable_folder.mkdir()
        tmp_metadata_file = mltable_folder / "MLTable"

        file_contents = """
            paths:
                - file: ./tmp_file.csv
            transformations:
                - read_delimited:
                    delimiter: ","
                    encoding: ascii
                    header: all_files_same_headers
        """
        tmp_metadata_file.write_text(file_contents)
        valid_metadata_dict = load_yaml(tmp_metadata_file)
        validate_mltable_metadata(mltable_metadata_dict=valid_metadata_dict, mltable_schema=mltable_schema)
        # no errors raised

        file_contents = """
            transformations:
                - read_delimited:
                    delimiter: ","
                    encoding: ascii
                    header: all_files_same_headers
        """
        tmp_metadata_file.write_text(file_contents)
        missing_paths_dict = load_yaml(tmp_metadata_file)
        with pytest.raises(ValidationException) as ex:
            validate_mltable_metadata(mltable_metadata_dict=missing_paths_dict, mltable_schema=mltable_schema)
        assert "'paths' is a required property" in ex.value.message

        file_contents = """
            paths:
                - file: ./tmp_file.csv
            transformations:
                - read_delimited:
                    delimiter: ","
                    encoding: unknownencoding
                    header: all_files_same_headers
        """
        tmp_metadata_file.write_text(file_contents)
        invalid_encoding_dict = load_yaml(tmp_metadata_file)
        with pytest.raises(ValidationException) as ex:
            validate_mltable_metadata(mltable_metadata_dict=invalid_encoding_dict, mltable_schema=mltable_schema)
        assert "unknownencoding" in ex.value.message



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from .automl_tabular import AutoMLTabular
from .classification_job import ClassificationJob
from .featurization_settings import ColumnTransformer, TabularFeaturizationSettings
from .forecasting_job import ForecastingJob
from .forecasting_settings import ForecastingSettings
from .limit_settings import TabularLimitSettings
from .regression_job import RegressionJob

__all__ = [
    "AutoMLTabular",
    "ClassificationJob",
    "ColumnTransformer",
    "ForecastingJob",
    "ForecastingSettings",
    "RegressionJob",
    "TabularFeaturizationSettings",
    "TabularLimitSettings",
]
